@@ -1,12 +1,15 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <winsock2.h>
+#include <ws2tcpip.h>
+
 extern "C" {
 #include <afs/param.h>
 #include <afs/stds.h>
@@ -135,10 +138,10 @@ BOOL CALLBACK Columns_DlgProc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
       return TRUE;
 
    if (msg == WM_INITDIALOG)
-      SetWindowLong (hDlg, DWL_USER, ((LPPROPSHEETPAGE)lp)->lParam);
+      SetWindowLongPtr (hDlg, DWLP_USER, ((LPPROPSHEETPAGE)lp)->lParam);
 
    ShowColumnsParams *pscp;
-   pscp = (ShowColumnsParams *)GetWindowLong (hDlg, DWL_USER);
+   pscp = (ShowColumnsParams *)GetWindowLongPtr (hDlg, DWLP_USER);
 
    switch (msg)
       {
@@ -306,7 +309,7 @@ void Columns_OnSelShown (HWND hDlg)
       }
    else
       {
-      int iAvail = LB_GetData (hList, ii);
+      int iAvail = (int) LB_GetData (hList, ii);
       EnableWindow (GetDlgItem (hDlg, IDC_COL_DELETE), (iAvail != 0));
 
       int ci = (int)SendMessage (hList, LB_GETCOUNT, 0, 0);
