@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,12 +9,18 @@
 
 #include <afxpriv.h>
 #include "stdafx.h"
+#include "PropFile.h"
+#include "PropACL.h"
+#include "PropVolume.h"
 #include <winsock2.h>
 #include <ws2tcpip.h>
+#include <shtypes.h>
+#include <shlwapi.h>
 
 extern "C" {
 #include <afs/param.h>
 #include <afs/stds.h>
+#include <afs/afs_consts.h>
 }
 
 #include <atlconv.h>
@@ -39,16 +45,18 @@ extern "C" {
 static char THIS_FILE[] = __FILE__;
 #endif
 
+#pragma comment(linker, "\"/manifestdependency:type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")
+
 
 ULONG nCMRefCount = 0;	// IContextMenu ref count
 ULONG nSERefCount = 0;	// IShellExtInit ref count
 ULONG nICRefCount=0;
 ULONG nTPRefCount=0;
 ULONG nXPRefCount=0;
+UINT nPSRefCount=0;
 
-#define MAXSIZE 2048 /* most I'll get back from PIOCTL */
 #define PCCHAR(str)	((char *)(const char *)str)
-static char space[MAXSIZE];
+static char space[AFS_PIOCTL_MAXSIZE];
 
 static BOOL IsADir(const CString& strName)
 {
@@ -64,6 +72,7 @@ static BOOL IsADir(const CString& strName)
 // CShellExt
 
 IMPLEMENT_DYNCREATE(CShellExt, CCmdTarget)
+IMPLEMENT_DYNCREATE(CShellExt2, CCmdTarget)
 #define REG_CLIENT_PARMS_KEY    "SYSTEM\\CurrentControlSet\\Services\\TransarcAFSDaemon\\Parameters"
 #define OVERLAYENABLED 1
 
@@ -75,6 +84,7 @@ CShellExt::CShellExt()
     HRESULT hr;
     UINT code;
     DWORD ShellOption,LSPsize,LSPtype;
+    m_overlayObject = 0;
     hr = SHGetMalloc(&m_pAlloc);
     m_bIsOverlayEnabled=FALSE;
     if (FAILED(hr))
@@ -85,12 +95,19 @@ CShellExt::CShellExt()
 			  &LSPtype, (LPBYTE)&ShellOption, &LSPsize);
     RegCloseKey (NPKey);
     m_bIsOverlayEnabled=((code==0) && (LSPtype==REG_DWORD) && ((ShellOption & OVERLAYENABLED)!=0));
+
+    INITCOMMONCONTROLSEX used = {
+        sizeof(INITCOMMONCONTROLSEX),
+        ICC_DATE_CLASSES | ICC_WIN95_CLASSES | ICC_BAR_CLASSES | ICC_USEREX_CLASSES
+    };
+    InitCommonControlsEx(&used);
+
     TRACE("Create CShellExt, Ref count %d/n",nCMRefCount);
 }
 
 CShellExt::~CShellExt()
 {
-    if(m_pAlloc) 
+    if(m_pAlloc)
 	m_pAlloc->Release();
     nCMRefCount--;
     TRACE("Destroy CShellExt, Ref count %d/n",nCMRefCount);
@@ -105,7 +122,7 @@ void CShellExt::OnFinalRelease()
     // object before calling the base class.
 
     CCmdTarget::OnFinalRelease();
-}	
+}
 
 
 BEGIN_MESSAGE_MAP(CShellExt, CCmdTarget)
@@ -121,7 +138,7 @@ BEGIN_DISPATCH_MAP(CShellExt, CCmdTarget)
 END_DISPATCH_MAP()
 
 // Note: we add support for IID_IShellExt to support typesafe binding
-//  from VBA.  This IID must match the GUID that is attached to the 
+//  from VBA.  This IID must match the GUID that is attached to the
 //  dispinterface in the .ODL file.
 
 #ifndef _WIN64
@@ -129,11 +146,15 @@ END_DISPATCH_MAP()
 // {DC515C27-6CAC-11D1-BAE7-00C04FD140D2}
 static const IID IID_IShellExt =
 { 0xdc515c27, 0x6cac, 0x11d1, { 0xba, 0xe7, 0x0, 0xc0, 0x4f, 0xd1, 0x40, 0xd2 } };
+static const IID IID_IShellExt2 =
+{ 0xdc515c27, 0x6cac, 0x11d1, { 0xba, 0xe7, 0x0, 0xc0, 0x4f, 0xd1, 0x40, 0xd3 } };
 #else
 // 64-bit
 // {5f820ca1-3dde-11db-b2ce-001558092db5}
 static const IID IID_IShellExt =
 { 0x5f820ca1, 0x3dde, 0x11db, {0xb2, 0xce, 0x00, 0x15, 0x58, 0x09, 0x2d, 0xb5 } };
+static const IID IID_IShellExt2 =
+{ 0x5f820ca1, 0x3dde, 0x11db, {0xb2, 0xce, 0x00, 0x15, 0x58, 0x09, 0x2d, 0xb6 } };
 #endif
 
 BEGIN_INTERFACE_MAP(CShellExt, CCmdTarget)
@@ -143,14 +164,17 @@ BEGIN_INTERFACE_MAP(CShellExt, CCmdTarget)
     INTERFACE_PART(CShellExt, IID_IShellIconOverlayIdentifier, IconExt)
     INTERFACE_PART(CShellExt, IID_IQueryInfo , ToolTipExt)
     INTERFACE_PART(CShellExt, IID_IPersistFile , PersistFileExt)
+    INTERFACE_PART(CShellExt, IID_IShellPropSheetExt , PropertySheetExt)
 END_INTERFACE_MAP()
 
 #ifndef _WIN64
     // 32-bit
-IMPLEMENT_OLECREATE(CShellExt, STR_EXT_TITLE, 0xdc515c27, 0x6cac, 0x11d1, 0xba, 0xe7, 0x0, 0xc0, 0x4f, 0xd1, 0x40, 0xd2)
+IMPLEMENT_OLECREATE(CShellExt, _STR_EXT_TITLE, 0xdc515c27, 0x6cac, 0x11d1, 0xba, 0xe7, 0x0, 0xc0, 0x4f, 0xd1, 0x40, 0xd2)
+IMPLEMENT_OLECREATE(CShellExt2, _STR_EXT_TITLE, 0xdc515c27, 0x6cac, 0x11d1, 0xba, 0xe7, 0x0, 0xc0, 0x4f, 0xd1, 0x40, 0xd3)
 #else
     // 64-bit
-IMPLEMENT_OLECREATE(CShellExt, STR_EXT_TITLE, 0x5f820ca1, 0x3dde, 0x11db, 0xb2, 0xce, 0x0, 0x15, 0x58, 0x09, 0x2d, 0xb5)
+IMPLEMENT_OLECREATE(CShellExt, _STR_EXT_TITLE, 0x5f820ca1, 0x3dde, 0x11db, 0xb2, 0xce, 0x0, 0x15, 0x58, 0x09, 0x2d, 0xb5)
+IMPLEMENT_OLECREATE(CShellExt2, _STR_EXT_TITLE, 0x5f820ca1, 0x3dde, 0x11db, 0xb2, 0xce, 0x0, 0x15, 0x58, 0x09, 0x2d, 0xb6)
 #endif
 
 
@@ -194,24 +218,25 @@ STDMETHODIMP CShellExt::XMenuExt::QueryContextMenu(HMENU hMenu,UINT indexMenu,
     if (uFlags & CMF_VERBSONLY)
 	return MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL, (USHORT)0);
 
+    if (!pThis->m_bIsPathInAFS)
+	return MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL, (USHORT)0);
+
     // Check to see if there's already an AFS menu here; if so, remove it
     int nItemsNow = GetMenuItemCount (hMenu);
     CString strAfsItemText = GetMessageString(IDS_AFS_ITEM);
+    CString strDeleteText = GetMessageString(IDS_MENU_DELETE);
+    CString strCutText = GetMessageString(IDS_MENU_CUT);
     LPCTSTR pszAfsItemText = (LPCTSTR)strAfsItemText;
     for (int iItem = 0; iItem < nItemsNow; iItem++) {
 	TCHAR szItemText[256];
 	if (!GetMenuString (hMenu, iItem, szItemText, 256, MF_BYPOSITION))
 	    continue;
-	if (!lstrcmp (szItemText, pszAfsItemText)) {
+	if (lstrcmp (szItemText, pszAfsItemText)==0) {
 	    DeleteMenu (hMenu, iItem, MF_BYPOSITION);
 	    continue;
 	}
-	if ((!lstrcmp(szItemText,_T("&Delete")))&&(pThis->m_bIsSymlink)) {	/*this is a symlink - don't present a delete menu!*/
-	    DeleteMenu (hMenu, iItem, MF_BYPOSITION);
-	    continue;
-	}
-	if ((!lstrcmp(szItemText,_T("Cu&t")))&&(pThis->m_bIsSymlink)) {	/*same for cut*/
-	    DeleteMenu (hMenu, iItem, MF_BYPOSITION);
+	if (((lstrcmp(szItemText,strDeleteText)==0)||(lstrcmp(szItemText,strCutText)==0))&&((pThis->m_bIsSymlink)||(pThis->m_bIsMountpoint))) {
+        DeleteMenu (hMenu, iItem, MF_BYPOSITION);
 	    continue;
 	}
     }
@@ -249,7 +274,7 @@ STDMETHODIMP CShellExt::XMenuExt::QueryContextMenu(HMENU hMenu,UINT indexMenu,
     ::InsertMenu(hMountPointMenu, indexMountPointMenu++, MF_STRING | MF_BYPOSITION, idCmdFirst + IDM_MOUNTPOINT_MAKE, GetMessageString(IDS_MP_MAKE_ITEM));
     ::InsertMenu(hAfsMenu, indexAfsMenu++, MF_STRING | MF_BYPOSITION | MF_POPUP, (UINT)hMountPointMenu, GetMessageString(IDS_MOUNT_POINT_ITEM));
 
-    ::InsertMenu(hAfsMenu, indexAfsMenu++, MF_STRING | MF_BYPOSITION, idCmdFirst + IDM_FLUSH, GetMessageString(IDS_FLUSH_FILE_DIR_ITEM));	
+    ::InsertMenu(hAfsMenu, indexAfsMenu++, MF_STRING | MF_BYPOSITION, idCmdFirst + IDM_FLUSH, GetMessageString(IDS_FLUSH_FILE_DIR_ITEM));
     ::InsertMenu(hAfsMenu, indexAfsMenu++, MF_STRING | MF_BYPOSITION, idCmdFirst + IDM_FLUSH_VOLUME, GetMessageString(IDS_FLUSH_VOLUME_ITEM));
     ::InsertMenu(hAfsMenu, indexAfsMenu++, MF_STRING | MF_BYPOSITION, idCmdFirst + IDM_SHOW_SERVER, GetMessageString(IDS_SHOW_FILE_SERVERS_ITEM));
     ::InsertMenu(hAfsMenu, indexAfsMenu++, MF_STRING | MF_BYPOSITION, idCmdFirst + IDM_SHOWCELL, GetMessageString(IDS_SHOW_CELL_ITEM));
@@ -289,9 +314,9 @@ STDMETHODIMP CShellExt::XMenuExt::QueryContextMenu(HMENU hMenu,UINT indexMenu,
     // Add a separator after us
     ::InsertMenu (hMenu, indexMenu + indexShellMenu++, MF_STRING | MF_BYPOSITION | MF_SEPARATOR, 0, TEXT(""));
 
-    return MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL, 
+    return MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL,
 			 (USHORT)indexAfsMenu + indexVolPartMenu + indexMountPointMenu + indexShellMenu + indexSymbolicMenu);
-}	
+}
 
 STDMETHODIMP CShellExt::XMenuExt::InvokeCommand(LPCMINVOKECOMMANDINFO lpici)
 {
@@ -312,7 +337,7 @@ STDMETHODIMP CShellExt::XMenuExt::InvokeCommand(LPCMINVOKECOMMANDINFO lpici)
 	break;
     }
 
-    case IDM_ACL_SET:			{ 
+    case IDM_ACL_SET:			{
 	CSetAfsAcl dlg;
 	ASSERT(files.GetSize() == 1);
 	dlg.SetDir(files[0]);
@@ -331,7 +356,7 @@ STDMETHODIMP CShellExt::XMenuExt::InvokeCommand(LPCMINVOKECOMMANDINFO lpici)
 	CheckVolumes();
 	break;
 
-    case IDM_MOUNTPOINT_SHOW:	
+    case IDM_MOUNTPOINT_SHOW:
 	ListMount(files);
 	break;
 
@@ -350,15 +375,15 @@ STDMETHODIMP CShellExt::XMenuExt::InvokeCommand(LPCMINVOKECOMMANDINFO lpici)
 	break;
     }
 
-    case IDM_FLUSH:				
+    case IDM_FLUSH:
 	Flush(files);
 	break;
 
-    case IDM_FLUSH_VOLUME:		
+    case IDM_FLUSH_VOLUME:
 	FlushVolume(files);
 	break;
 
-    case IDM_SHOW_SERVER:	       
+    case IDM_SHOW_SERVER:
 	WhereIs(files);
 	break;
 
@@ -423,7 +448,7 @@ STDMETHODIMP CShellExt::XMenuExt::InvokeCommand(LPCMINVOKECOMMANDINFO lpici)
 	break;
     }
 
-    case IDM_SYMBOLICLINK_SHOW:	
+    case IDM_SYMBOLICLINK_SHOW:
 	ListSymlink(files);
 	break;
 
@@ -433,7 +458,7 @@ STDMETHODIMP CShellExt::XMenuExt::InvokeCommand(LPCMINVOKECOMMANDINFO lpici)
 	int nChoice = ShowMessageBox(IDS_REALLY_REMOVE_SYMLINK, MB_ICONQUESTION | MB_YESNO, IDS_REALLY_REMOVE_SYMLINK);
 	if (nChoice == IDYES)
 	    RemoveSymlink(files.GetAt(0));
-	}	
+	}
 	break;
     default:
 	ASSERT(FALSE);
@@ -444,7 +469,7 @@ STDMETHODIMP CShellExt::XMenuExt::InvokeCommand(LPCMINVOKECOMMANDINFO lpici)
     Release();
 
     return NOERROR;
-}	
+}
 
 STDMETHODIMP CShellExt::XMenuExt::GetCommandString(UINT_PTR idCmd, UINT uType,
     UINT* pwReserved, LPSTR pszName, UINT cchMax)
@@ -458,11 +483,11 @@ STDMETHODIMP CShellExt::XMenuExt::GetCommandString(UINT_PTR idCmd, UINT uType,
 
     switch (idCmd)
     {
-    case IDM_AUTHENTICATION: 
+    case IDM_AUTHENTICATION:
 	nCmdStrID = ID_AUTHENTICATE;
 	break;
 
-    case IDM_ACL_SET: 
+    case IDM_ACL_SET:
 	nCmdStrID = ID_ACL_SET;
 	break;
 
@@ -470,58 +495,58 @@ STDMETHODIMP CShellExt::XMenuExt::GetCommandString(UINT_PTR idCmd, UINT uType,
 	nCmdStrID = ID_VOLUME_PROPERTIES;
 	break;
 
-    case IDM_VOLUMEPARTITION_UPDATENAMEIDTABLE:	
+    case IDM_VOLUMEPARTITION_UPDATENAMEIDTABLE:
 	nCmdStrID = ID_VOLUMEPARTITION_UPDATENAMEIDTABLE;
 	break;
 
-    case IDM_MOUNTPOINT_SHOW:  
+    case IDM_MOUNTPOINT_SHOW:
 	nCmdStrID = ID_MOUNTPOINT_SHOW;
 	break;
 
-    case IDM_MOUNTPOINT_REMOVE:	
+    case IDM_MOUNTPOINT_REMOVE:
 	nCmdStrID = ID_MOUNTPOINT_REMOVE;
 	break;
-		
-    case IDM_MOUNTPOINT_MAKE: 
+
+    case IDM_MOUNTPOINT_MAKE:
 	nCmdStrID = ID_MOUNTPOINT_MAKE;
 	break;
 
-    case IDM_FLUSH:  
+    case IDM_FLUSH:
 	nCmdStrID = ID_FLUSH;
 	break;
 
-    case IDM_FLUSH_VOLUME:   
+    case IDM_FLUSH_VOLUME:
 	nCmdStrID = ID_VOLUME_FLUSH;
 	break;
 
-    case IDM_SHOW_SERVER: 
+    case IDM_SHOW_SERVER:
 	nCmdStrID = ID_WHEREIS;
 	break;
 
-    case IDM_SHOWCELL:	
+    case IDM_SHOWCELL:
 	nCmdStrID = ID_SHOWCELL;
 	break;
 
-    case IDM_SERVER_STATUS: 
+    case IDM_SERVER_STATUS:
 	nCmdStrID = ID_SERVER_STATUS;
 	break;
 
     case IDM_SYMBOLICLINK_ADD:
 	nCmdStrID = ID_SYMBOLICLINK_ADD;
 	break;
-		
+
     case IDM_SYMBOLICLINK_SHOW:
 	nCmdStrID = ID_SYMBOLICLINK_SHOW;
 	break;
 
-    case IDM_SYMBOLICLINK_REMOVE: 
+    case IDM_SYMBOLICLINK_REMOVE:
 	nCmdStrID = ID_SYMBOLICLINK_REMOVE;
 	break;
 
     case IDM_REMOVE_SYMLINK:
 	nCmdStrID= ID_REMOVE_SYMLINK;
 	break;
-		
+
     default:
 	ASSERT(FALSE);
 	Release();
@@ -555,7 +580,7 @@ STDMETHODIMP_(ULONG) CShellExt::XShellInit::Release(void)
 {
     if (nSERefCount > 0)
 	nSERefCount--;
-	
+
     return nSERefCount;
 }
 
@@ -571,57 +596,96 @@ STDMETHODIMP CShellExt::XShellInit::Initialize(LPCITEMIDLIST pidlFolder, IDataOb
     STGMEDIUM medium;
 
     // We must have a data object
-    if (pdobj == NULL)
-	    return E_FAIL;
+    if ((pdobj == NULL) && (pidlFolder == NULL))
+        return E_FAIL;
 
-    //  Use the given IDataObject to get a list of filenames (CF_HDROP)
-    hres = pdobj->GetData(&fmte, &medium);
-    if (FAILED(hres)) {
-	return E_FAIL;
+    pThis->m_bIsSymlink=false;
+    pThis->m_bIsMountpoint=false;
+    pThis->m_bIsPathInAFS=false;
+    pThis->m_bDirSelected=false;
+
+    if (pdobj) {
+        //  Use the given IDataObject to get a list of filenames (CF_HDROP)
+        hres = pdobj->GetData(&fmte, &medium);
+        if (FAILED(hres)) {
+        return E_FAIL;
+        }
+
+        int nNumFiles = DragQueryFile((HDROP)medium.hGlobal, 0xFFFFFFFF, NULL, 0);
+        if (nNumFiles == 0)
+            hres = E_FAIL;
+        else {
+            pThis->m_bDirSelected = FALSE;
+
+            for (int ii = 0; ii < nNumFiles; ii++) {
+                CString strFileName;
+
+                // Get the size of the file name string
+                int nNameLen = DragQueryFile((HDROP)medium.hGlobal, ii, 0, 0);
+
+                // Make room for it in our string object
+                LPTSTR pszFileNameBuf = strFileName.GetBuffer(nNameLen + 1);	// +1 for the terminating NULL
+                ASSERT(pszFileNameBuf);
+
+                // Get the file name
+                DragQueryFile((HDROP)medium.hGlobal, ii, pszFileNameBuf, nNameLen + 1);
+
+                strFileName.ReleaseBuffer();
+                if (!IsPathInAfs(strFileName)) {
+                pThis->m_astrFileNames.RemoveAll();
+                pThis->m_bIsPathInAFS=false;
+                break;
+                } else {
+                pThis->m_bIsSymlink=pThis->m_bIsSymlink||IsSymlink(strFileName);
+                pThis->m_bIsMountpoint=pThis->m_bIsMountpoint||IsMountPoint(strFileName);
+                pThis->m_bIsPathInAFS=true;
+                }
+
+                if (IsADir(strFileName))
+                pThis->m_bDirSelected = TRUE;
+
+                pThis->m_astrFileNames.Add(strFileName);
+            }
+            //	Release the data
+            ReleaseStgMedium(&medium);
+        }
     }
+    if ((pThis->m_astrFileNames.GetSize() == 0)&&(pidlFolder)) {
+        // if there are no valid files selected, try the folder background
+        IShellFolder *parentFolder = NULL;
+        STRRET name;
+        TCHAR * szDisplayName = NULL;
 
-    int nNumFiles = DragQueryFile((HDROP)medium.hGlobal, 0xFFFFFFFF, NULL, 0);
-    if (nNumFiles == 0)
-	hres = E_FAIL;
-    else {
-	pThis->m_bDirSelected = FALSE;
+        hres = ::SHGetDesktopFolder(&parentFolder);
+        if (FAILED(hres))
+            return hres;
 
-	for (int ii = 0; ii < nNumFiles; ii++) {
-	    CString strFileName;
-
-	    // Get the size of the file name string
-	    int nNameLen = DragQueryFile((HDROP)medium.hGlobal, ii, 0, 0);
-
-	    // Make room for it in our string object
-	    LPTSTR pszFileNameBuf = strFileName.GetBuffer(nNameLen + 1);	// +1 for the terminating NULL
-	    ASSERT(pszFileNameBuf);
-
-	    // Get the file name
-	    DragQueryFile((HDROP)medium.hGlobal, ii, pszFileNameBuf, nNameLen + 1);
-
-	    strFileName.ReleaseBuffer();
-
-	    if (!IsPathInAfs(strFileName)) {
-		pThis->m_astrFileNames.RemoveAll();
-		break;
-	    } else {
-		pThis->m_bIsSymlink=IsSymlink(strFileName);
-	    }
-
-	    if (IsADir(strFileName))
-		pThis->m_bDirSelected = TRUE;
-
-	    pThis->m_astrFileNames.Add(strFileName);
-	}
+        hres = parentFolder->GetDisplayNameOf(pidlFolder, SHGDN_NORMAL | SHGDN_FORPARSING, &name);
+        if (FAILED(hres)) {
+            parentFolder->Release();
+            return hres;
+        }
 
+        hres = StrRetToStr (&name, pidlFolder, &szDisplayName);
+        if (FAILED(hres))
+            return hres;
+        parentFolder->Release();
+        if (szDisplayName) {
+            pThis->m_bDirSelected = TRUE;
+            CString strFileName = CString(szDisplayName);
+            if (IsPathInAfs(strFileName)) {
+                pThis->m_bIsSymlink=IsSymlink(strFileName);
+                pThis->m_bIsMountpoint=IsMountPoint(strFileName);
+                pThis->m_bIsPathInAFS=true;
+                pThis->m_astrFileNames.Add(strFileName);
+            }
+            CoTaskMemFree(szDisplayName);
+        }
+    }
 	if (pThis->m_astrFileNames.GetSize() > 0)
 	    hres = NOERROR;
 	else
 	    hres = E_FAIL;
-    }
- 
-    //	Release the data
-    ReleaseStgMedium(&medium);
 
     return hres;
 }
@@ -643,7 +707,7 @@ STDMETHODIMP_(ULONG) CShellExt::XIconExt::Release(void)
 	nICRefCount--;
 
     return nICRefCount;
-}	
+}
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -653,21 +717,35 @@ STDMETHODIMP_(ULONG) CShellExt::XIconExt::Release(void)
 STDMETHODIMP CShellExt::XIconExt::GetOverlayInfo(LPWSTR pwszIconFile
 	,int cchMax,int* pIndex,DWORD* pdwFlags)
 {
+    METHOD_PROLOGUE(CShellExt, IconExt);
     if(IsBadWritePtr(pIndex, sizeof(int)))
 	return E_INVALIDARG;
     if(IsBadWritePtr(pdwFlags, sizeof(DWORD)))
 	return E_INVALIDARG;
 
-    HMODULE hModule=GetModuleHandle(_T("shell32.dll"));
+    // The icons must reside in the same path as this dll
     TCHAR szModule[MAX_PATH];
-    DWORD z=GetModuleFileName(hModule,szModule,sizeof(szModule));
+    GetModuleFileName(theApp.m_hInstance, szModule, MAX_PATH);
+    TCHAR * slash = _tcsrchr(szModule, '\\');
+    if (slash) {
+        *slash = 0;
+        switch (pThis->GetOverlayObject())
+        {
+            case 0:
+                _tcscat(szModule, _T("\\link.ico"));
+            break;
+            case 1:
+                _tcscat(szModule, _T("\\mount.ico"));
+            break;
+        }
+    }
 #ifndef UNICODE
-    MultiByteToWideChar( CP_ACP,0,szModule,-1,pwszIconFile,cchMax); 
+    MultiByteToWideChar( CP_ACP,0,szModule,-1,pwszIconFile,cchMax);
 #else
     _tcsncpy(pwszIconFile, szModule, cchMax);
 #endif
-    *pIndex = 30;
-    *pdwFlags = ISIOI_ICONFILE | ISIOI_ICONINDEX;
+    *pIndex = 0;
+    *pdwFlags = ISIOI_ICONFILE;
     return S_OK;
 }
 
@@ -681,16 +759,24 @@ STDMETHODIMP CShellExt::XIconExt::GetPriority(int* pPriority)
 
 STDMETHODIMP CShellExt::XIconExt::IsMemberOf(LPCWSTR pwszPath,DWORD dwAttrib)
 {
+    METHOD_PROLOGUE(CShellExt, IconExt);
     TCHAR szPath[MAX_PATH];
 #ifdef UNICODE
     _tcscpy(szPath, pwszPath);
 #else
     WideCharToMultiByte( CP_ACP,0,pwszPath,-1,szPath,MAX_PATH,NULL,NULL);
 #endif
-    if (IsSymlink(szPath))
-	return S_OK;
+	if (!IsPathInAfs(szPath))
+		return S_FALSE;
+
+    if ((pThis->GetOverlayObject() == 0)&&(IsSymlink(szPath))) {
+        return S_OK;
+    }
+    if ((pThis->GetOverlayObject() == 1)&&(IsMountPoint(szPath))) {
+        return S_OK;
+    }
     return S_FALSE;
-}	
+}
 
 /*  TOOL TIP INFO IMPLIMENTION   */
 
@@ -717,23 +803,34 @@ STDMETHODIMP CShellExt::XToolTipExt::GetInfoTip(DWORD dwFlags, LPWSTR *ppwszTip)
 {
     METHOD_PROLOGUE(CShellExt, ToolTipExt);
 
-    if (!IsSymlink(pThis->m_szFile))
+    if ((_tcslen(pThis->m_szFile) == 0)||(!IsPathInAfs(pThis->m_szFile)))
     {
-	ppwszTip=NULL;
+        *ppwszTip=NULL;
+        return S_OK;
+    }
+    bool bIsSymlink = !!IsSymlink(pThis->m_szFile);
+    bool bIsMountpoint = !!IsMountPoint(pThis->m_szFile);
+    if ((!bIsSymlink) && (!bIsMountpoint))
+    {
+	*ppwszTip=NULL;
 	return S_OK;
     }
     USES_CONVERSION;
     // dwFlags is currently unused.
-    *ppwszTip = (WCHAR*) (pThis->m_pAlloc)->Alloc((1+lstrlen(pThis->m_szFile))*sizeof(WCHAR));
+    CString sInfo;
+    if (bIsSymlink)
+        sInfo = GetSymlink(pThis->m_szFile);
+    else if (bIsMountpoint)
+        sInfo = GetMountpoint(pThis->m_szFile);
+    *ppwszTip = (WCHAR*) (pThis->m_pAlloc)->Alloc((1+sInfo.GetLength())*sizeof(WCHAR));
     if (*ppwszTip)
-    {
-	wcscpy(*ppwszTip, (WCHAR*)T2OLE(pThis->m_szFile));
-    }
+        wcscpy(*ppwszTip, (WCHAR*)T2COLE((LPCTSTR)sInfo));
 
     return S_OK;
 }
 STDMETHODIMP CShellExt::XToolTipExt::GetInfoFlags(LPDWORD pdwFlags)
 {
+    *pdwFlags = 0;
     return S_OK;
 }
 
@@ -763,31 +860,212 @@ STDMETHODIMP	CShellExt::XPersistFileExt::Load(LPCOLESTR wszFile, DWORD dwMode)
 {
     METHOD_PROLOGUE(CShellExt, PersistFileExt);
     USES_CONVERSION;
-    _tcscpy(pThis->m_szFile, OLE2T((WCHAR*)wszFile)); 
-    return S_OK;	
+    _tcscpy(pThis->m_szFile, OLE2T((WCHAR*)wszFile));
+    return S_OK;
 }
 
 STDMETHODIMP CShellExt::XPersistFileExt::GetClassID(LPCLSID)
-{ 
-    return E_NOTIMPL;	
+{
+    return E_NOTIMPL;
 }
 
 STDMETHODIMP CShellExt::XPersistFileExt::IsDirty(VOID)
-{ 
-    return E_NOTIMPL; 
+{
+    return E_NOTIMPL;
 }
 
 STDMETHODIMP CShellExt::XPersistFileExt::Save(LPCOLESTR, BOOL)
-{ 
-    return E_NOTIMPL; 
+{
+    return E_NOTIMPL;
 }
 
 STDMETHODIMP CShellExt::XPersistFileExt::SaveCompleted(LPCOLESTR)
-{ 
-    return E_NOTIMPL; 
+{
+    return E_NOTIMPL;
 }
 
 STDMETHODIMP CShellExt::XPersistFileExt::GetCurFile(LPOLESTR FAR*)
-{ 
-    return E_NOTIMPL; 
+{
+    return E_NOTIMPL;
+}
+
+// IShellPropSheetExt
+STDMETHODIMP CShellExt::XPropertySheetExt::QueryInterface(REFIID riid, void** ppv)
+{
+    METHOD_PROLOGUE(CShellExt, PropertySheetExt);
+    return pThis->ExternalQueryInterface(&riid, ppv);
+}
+
+STDMETHODIMP_(ULONG) CShellExt::XPropertySheetExt::AddRef(void)
+{
+    return ++nPSRefCount;
+}
+
+STDMETHODIMP_(ULONG) CShellExt::XPropertySheetExt::Release(void)
+{
+    if (nPSRefCount> 0)
+	nPSRefCount--;
+
+    return nPSRefCount;
+}
+
+BOOL CALLBACK PageProc (HWND hwnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
+{
+    PropertyPage * sheetpage;
+
+    if (uMessage == WM_INITDIALOG)
+    {
+        sheetpage = (PropertyPage*) ((LPPROPSHEETPAGE) lParam)->lParam;
+        SetWindowLongPtr (hwnd, GWLP_USERDATA, (LONG_PTR) sheetpage);
+        sheetpage->SetHwnd(hwnd);
+    }
+    else
+    {
+        sheetpage = (PropertyPage*) GetWindowLongPtr (hwnd, GWLP_USERDATA);
+    }
+
+    if (sheetpage != 0L)
+        return sheetpage->PropPageProc(hwnd, uMessage, wParam, lParam);
+    else
+        return FALSE;
+}
+
+UINT CALLBACK PropPageCallbackProc ( HWND /*hwnd*/, UINT uMsg, LPPROPSHEETPAGE ppsp )
+{
+    // Delete the page before closing.
+    if (PSPCB_CREATE == uMsg)
+        return TRUE;
+    if (PSPCB_RELEASE == uMsg)
+    {
+        PropertyPage* sheetpage = (PropertyPage*) ppsp->lParam;
+        delete sheetpage;
+    }
+    return TRUE;
+}
+
+STDMETHODIMP CShellExt::XPropertySheetExt::AddPages(LPFNADDPROPSHEETPAGE lpfnAddPage, LPARAM lParam)
+{
+    METHOD_PROLOGUE(CShellExt, PropertySheetExt);
+
+    if(pThis->m_bIsPathInAFS) {
+        // add the property page for files/folder/mount points/symlinks
+        PROPSHEETPAGE psp;
+        SecureZeroMemory(&psp, sizeof(PROPSHEETPAGE));
+        HPROPSHEETPAGE hPage;
+        CPropFile *sheetpage = NULL;
+
+        sheetpage = new CPropFile(pThis->m_astrFileNames);
+
+        if (sheetpage == NULL)
+            return E_OUTOFMEMORY;
+
+        HINSTANCE hInst = 0;
+        TaLocale_GetResource(RT_DIALOG, MAKEINTRESOURCE(IDD_PROPPAGE_FILE), LANG_USER_DEFAULT, &hInst);
+        sheetpage->m_hInst = hInst;
+        sheetpage->m_bIsMountpoint = pThis->m_bIsMountpoint;
+        sheetpage->m_bIsSymlink = pThis->m_bIsSymlink;
+        sheetpage->m_bIsDir=pThis->m_bDirSelected;
+        psp.dwSize = sizeof (psp);
+        psp.dwFlags = PSP_USEREFPARENT | PSP_USETITLE | PSP_USECALLBACK | PSP_USETITLE;
+        psp.hInstance = hInst;
+        psp.pszTemplate = MAKEINTRESOURCE(IDD_PROPPAGE_FILE);
+        psp.pszIcon = NULL;
+        psp.pszTitle = _T("AFS");
+        psp.pfnDlgProc = (DLGPROC) PageProc;
+        psp.lParam = (LPARAM) sheetpage;
+        psp.pfnCallback = PropPageCallbackProc;
+        psp.pcRefParent = (UINT*) &nPSRefCount;
+
+        hPage = CreatePropertySheetPage (&psp);
+
+        if (hPage != NULL) {
+            if (!lpfnAddPage (hPage, lParam)) {
+                delete sheetpage;
+                DestroyPropertySheetPage (hPage);
+            }
+        }
+    }
+
+    // add the property page for Volume Data
+    PROPSHEETPAGE psp;
+    SecureZeroMemory(&psp, sizeof(PROPSHEETPAGE));
+    HPROPSHEETPAGE hPage;
+    CPropVolume *sheetpage = NULL;
+
+    sheetpage = new CPropVolume(pThis->m_astrFileNames);
+
+    if (sheetpage == NULL)
+        return E_OUTOFMEMORY;
+
+    HINSTANCE hInst = 0;
+    TaLocale_GetResource(RT_DIALOG, MAKEINTRESOURCE(IDD_PROPPAGE_VOLUME), LANG_USER_DEFAULT, &hInst);
+    sheetpage->m_hInst = hInst;
+    sheetpage->m_bIsMountpoint = pThis->m_bIsMountpoint;
+    sheetpage->m_bIsSymlink = pThis->m_bIsSymlink;
+    sheetpage->m_bIsDir=pThis->m_bDirSelected;
+    psp.dwSize = sizeof (psp);
+    psp.dwFlags = PSP_USEREFPARENT | PSP_USETITLE | PSP_USECALLBACK | PSP_USETITLE;
+    psp.hInstance = hInst;
+    psp.pszTemplate = MAKEINTRESOURCE(IDD_PROPPAGE_VOLUME);
+    psp.pszIcon = NULL;
+    psp.pszTitle = _T("AFS Volume");
+    psp.pfnDlgProc = (DLGPROC) PageProc;
+    psp.lParam = (LPARAM) sheetpage;
+    psp.pfnCallback = PropPageCallbackProc;
+    psp.pcRefParent = (UINT*) &nPSRefCount;
+
+    hPage = CreatePropertySheetPage (&psp);
+
+    if (hPage != NULL) {
+        if (!lpfnAddPage (hPage, lParam)) {
+            delete sheetpage;
+            DestroyPropertySheetPage (hPage);
+        }
+    }
+
+    if(pThis->m_bDirSelected) {
+        // add the property page for ACLs
+        PROPSHEETPAGE psp;
+        SecureZeroMemory(&psp, sizeof(PROPSHEETPAGE));
+        HPROPSHEETPAGE hPage;
+        CPropACL *sheetpage = NULL;
+
+        sheetpage = new CPropACL(pThis->m_astrFileNames);
+
+        if (sheetpage == NULL)
+            return E_OUTOFMEMORY;
+
+        HINSTANCE hInst = 0;
+        TaLocale_GetResource(RT_DIALOG, MAKEINTRESOURCE(IDD_PROPPAGE_ACL), LANG_USER_DEFAULT, &hInst);
+        sheetpage->m_hInst = hInst;
+        sheetpage->m_bIsMountpoint = pThis->m_bIsMountpoint;
+        sheetpage->m_bIsSymlink = pThis->m_bIsSymlink;
+        sheetpage->m_bIsDir=pThis->m_bDirSelected;
+        psp.dwSize = sizeof (psp);
+        psp.dwFlags = PSP_USEREFPARENT | PSP_USETITLE | PSP_USECALLBACK | PSP_USETITLE;
+        psp.hInstance = hInst;
+        psp.pszTemplate = MAKEINTRESOURCE(IDD_PROPPAGE_ACL);
+        psp.pszIcon = NULL;
+        psp.pszTitle = _T("AFS ACL");
+        psp.pfnDlgProc = (DLGPROC) PageProc;
+        psp.lParam = (LPARAM) sheetpage;
+        psp.pfnCallback = PropPageCallbackProc;
+        psp.pcRefParent = (UINT*) &nPSRefCount;
+
+        hPage = CreatePropertySheetPage (&psp);
+
+        if (hPage != NULL) {
+            if (!lpfnAddPage (hPage, lParam)) {
+                delete sheetpage;
+                DestroyPropertySheetPage (hPage);
+            }
+        }
+    }
+
+    return S_OK;
+}
+
+STDMETHODIMP CShellExt::XPropertySheetExt::ReplacePage(UINT uPageID, LPFNADDPROPSHEETPAGE pfnReplacePage, LPARAM lParam)
+{
+    return E_FAIL;
 }
