@@ -1,22 +1,20 @@
-/* 
- * Copyright (C) 1998, 1989 Transarc Corporation - All rights reserved
- *
- * (C) COPYRIGHT IBM CORPORATION 1987, 1988
- * LICENSED MATERIALS - PROPERTY OF IBM
+/*
+ * Copyright 2000, International Business Machines Corporation and others.
+ * All Rights Reserved.
  *
+ * This software has been released under the terms of the IBM Public
+ * License.  For details, see the LICENSE file in the top-level source
+ * directory or online at http://www.openafs.org/dl/license10.html
  */
 
 /* Copyright (C) 1994 Cazamar Systems, Inc. */
 
-#ifndef _OSISLEEP_H_ENV_
-#define _OSISLEEP_H_ENV_ 1
+#ifndef OPENAFS_WINNT_CLIENT_OSI_OSISLEEP_H
+#define OPENAFS_WINNT_CLIENT_OSI_OSISLEEP_H 1
 
 /*#include "osi.h"*/
 #include "osifd.h"
 #include "osiqueue.h"
-#ifdef DJGPP
-#include "osithrd95.h"
-#endif /* DJGPP */
 
 /* states bits */
 #define OSI_SLEEPINFO_SIGNALLED	1	/* this sleep structure has been signalled */
@@ -27,14 +25,15 @@
 #define OSI_SLEEPINFO_W4READ	1	/* waiting for a read lock */
 #define OSI_SLEEPINFO_W4WRITE	2	/* waiting for a write lock */
 typedef struct osi_sleepInfo {
-	osi_queue_t q;
-	long value;		/* sleep value when in a sleep queue, patch addr for turnstiles */
-	unsigned long tid;	/* thread ID of sleeper */
-	EVENT_HANDLE sema;		/* semaphore for this entry */
-	unsigned short states;	/* states bits */
-	unsigned short idx;	/* sleep hash table we're in, if in hash */
-        unsigned short waitFor;	/* what are we waiting for; used for bulk wakeups */
-	unsigned short refCount;/* reference count from FDs */
+    osi_queue_t q;
+    LONG_PTR value;		/* sleep value when in a sleep queue, patch addr for turnstiles */
+    DWORD *tidp;                /* tid history */
+    DWORD tid;		        /* thread ID of sleeper */
+    EVENT_HANDLE sema;	        /* semaphore for this entry */
+    unsigned short states;	/* states bits */
+    unsigned short idx;	        /* sleep hash table we're in, if in hash */
+    unsigned short waitFor;	/* what are we waiting for; used for bulk wakeups */
+    unsigned long refCount;     /* reference count from FDs */
 } osi_sleepInfo_t;
 
 /* first guy is the most recently added process */
@@ -51,18 +50,14 @@ typedef struct osi_sleepFD{
 
 /* struct for single-shot initialization support */
 typedef struct osi_once {
-#ifndef DJGPP
 	long atomic;	/* used for atomicity */
-#else
-	osi_mutex_t atomic;	/* used for atomicity */
-#endif /* !DJGPP */
 	int done;	/* tells if initialization is done */
 } osi_once_t;
 
 /* size of mutex hash table; should be a prime number; used for mutex and lock hashing */
 #define OSI_MUTEXHASHSIZE	251	/* prime number */
 
-#define osi_MUTEXHASH(x) ((unsigned short) (((unsigned long) x) % (unsigned) OSI_MUTEXHASHSIZE))
+#define osi_MUTEXHASH(x) ((unsigned short) (((LONG_PTR) x) % (intptr_t) OSI_MUTEXHASHSIZE))
 
 /* size of sleep value hash table.  Must be power of 2 */
 #define OSI_SLEEPHASHSIZE	128
@@ -77,13 +72,13 @@ typedef struct osi_once {
 extern Crit_Sec osi_sleepCookieCS;
 
 /* spin lock version of atomic sleep, used internally only */
-extern void osi_SleepSpin(long value, Crit_Sec *counterp);
+extern void osi_SleepSpin(LONG_PTR value, Crit_Sec *counterp);
 
 /* spin lock version of wakeup, used internally only */
-extern void osi_WakeupSpin(long value);
+extern void osi_WakeupSpin(LONG_PTR value);
 
 /* exported function to sleep on a value */
-extern void osi_Sleep (long);
+extern void osi_Sleep (LONG_PTR);
 
 extern void osi_FreeSleepInfo(osi_sleepInfo_t *);
 
@@ -108,12 +103,10 @@ extern int osi_TestOnce(osi_once_t *);
 extern void osi_EndOnce(osi_once_t *);
 
 
-#ifndef DJGPP
 /* exported function to wakeup those sleeping on a value */
-extern void osi_Wakeup (long);
+extern void osi_Wakeup (LONG_PTR);
 
 extern void osi_Init (void);
-#endif /* !DJGPP */
 
 /* create a ptr to a cookie */
 osi_sleepFD_t *osi_CreateSleepCookie(void);
@@ -126,9 +119,7 @@ int osi_NextSleepCookie(osi_sleepFD_t *);
 
 /* functions for the sleep FD implementation */
 extern long osi_SleepFDCreate(osi_fdType_t *, osi_fd_t **);
-#ifndef DJGPP
 extern long osi_SleepFDGetInfo(osi_fd_t *, osi_remGetInfoParms_t *);
-#endif
 extern long osi_SleepFDClose(osi_fd_t *);
 
 /* functions for getting hash sizes */
@@ -152,10 +143,15 @@ unsigned long osi_GetBootTime(void);
 void osi_InitPanic(void *anotifFunc);
 void osi_panic(char *, char *, long);
 
-unsigned long osi_Time(void);
+time_t osi_Time(void);
 
-extern void osi_TWait(osi_turnstile_t *turnp, int waitFor, void *patchp,
-	Crit_Sec *releasep);
+extern void osi_TWait(osi_turnstile_t *turnp, int waitFor,
+                      void *patchp, DWORD *tidp,
+                      Crit_Sec *releasep);
+
+extern void osi_TWaitExt(osi_turnstile_t *turnp, int waitFor,
+                         void *patchp, DWORD *tidp,
+                         Crit_Sec *releasep, int prepend);
 
 extern void osi_TSignal(osi_turnstile_t *turnp);
 
@@ -167,4 +163,4 @@ extern void osi_TSignalForMLs(osi_turnstile_t *turnp, int stillHaveReaders, Crit
 
 #define osi_TEmpty(t)	((t)->firstp == NULL)
 
-#endif /*_OSISLEEP_H_ENV_ */
+#endif /* OPENAFS_WINNT_CLIENT_OSI_OSISLEEP_H */
