@@ -27,6 +27,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
+
 #include<windows.h>
 #include<netidmgr.h>
 #include<dynimport.h>
@@ -53,8 +55,8 @@ khm_krb5_error(krb5_error_code rc, LPCSTR FailedFunctionName,
     const char *errText;
     int krb5Error = ((int)(rc & 255));
 
-    if (*ctx && pkrb5_get_error_message)
-        errText = pkrb5_get_error_message(*ctx, rc);
+    if (pkrb5_get_error_message)
+        errText = pkrb5_get_error_message(rc);
     else
         errText = perror_message(rc);
     _snprintf(message, sizeof(message),
@@ -62,8 +64,8 @@ khm_krb5_error(krb5_error_code rc, LPCSTR FailedFunctionName,
         errText,
         krb5Error,
         FailedFunctionName);
-    if (*ctx && pkrb5_free_error_message)
-        pkrb5_free_error_message(*ctx, errText);
+    if (pkrb5_free_error_message)
+        pkrb5_free_error_message(errText);
 
     MessageBoxA(NULL, message, "Kerberos Five", MB_OK | MB_ICONERROR |
         MB_TASKMODAL |
