@@ -20,6 +20,8 @@
 #endif
 
 #ifdef AFS_DARWIN80_ENV
+static thread_t vfs_context_owner;
+
 /* works like PFlushVolumeData */
 void
 darwin_notify_perms(struct unixuser *auser, int event)
@@ -30,6 +32,7 @@ darwin_notify_perms(struct unixuser *auser, int event)
     int isglock = ISAFS_GLOCK();
     struct vnode *vp;
     struct vnode_attr va;
+    int isctxtowner = 0;
 
     if (!afs_darwin_fsevents)
 	return;
@@ -41,9 +44,12 @@ darwin_notify_perms(struct unixuser *auser, int event)
     else
 	VATTR_SET(&va, va_uid, -2); /* nobody */
 
-    get_vfs_context();
     if (!isglock)
 	AFS_GLOCK();
+    if (!(vfs_context_owner == current_thread())) {
+	get_vfs_context();
+	isctxtowner = 1;
+    }
 loop:
     ObtainReadLock(&afs_xvcache);
     for (i = 0; i < VCSIZE; i++) {
@@ -58,7 +64,7 @@ loop:
 	    if (!(vp = AFSTOV(tvc)) || !vnode_isdir(AFSTOV(tvc)))
 		continue;
 	    /* dynroot object. no callbacks. anonymous ACL. just no. */
-	    if (afs_IsDynrootFid(tvc))
+	    if (afs_IsDynrootFid(&tvc->f.fid))
 		continue;
 	    /* no fake fsevents on mount point sources. leaks refs */
 	    if (tvc->mvstat == 1)
@@ -73,23 +79,27 @@ loop:
 		continue;
 	    }
 	    ReleaseReadLock(&afs_xvcache);
-	    ObtainWriteLock(&tvc->lock, 234);
-	    tvc->f.states |= CEvent;
-	    AFS_GUNLOCK();
-	    vnode_setattr(vp, &va, afs_osi_ctxtp);
-	    tvc->f.states &= ~CEvent;
-	    vnode_put(vp);
-	    AFS_GLOCK();
-	    ReleaseWriteLock(&tvc->lock);
+	    /* Avoid potentially re-entering on this lock */
+	    if (0 == NBObtainWriteLock(&tvc->lock, 234)) {
+		tvc->f.states |= CEvent;
+		AFS_GUNLOCK();
+		vnode_setattr(vp, &va, afs_osi_ctxtp);
+		tvc->f.states &= ~CEvent;
+		vnode_put(vp);
+		AFS_GLOCK();
+		ReleaseWriteLock(&tvc->lock);
+	    }
 	    ObtainReadLock(&afs_xvcache);
+	    uq = QPrev(tq);
 	    /* our tvc ptr is still good until now */
 	    AFS_FAST_RELE(tvc);
 	}
     }
     ReleaseReadLock(&afs_xvcache);
+    if (isctxtowner)
+	put_vfs_context();
     if (!isglock)
 	AFS_GUNLOCK();
-    put_vfs_context();
 }
 
 int
@@ -175,10 +185,10 @@ afs_suser(void *credp)
 }
 
 #ifdef AFS_DARWIN80_ENV
-uio_t
-afsio_darwin_partialcopy(uio_t auio, int size)
+struct uio *
+afsio_partialcopy(struct uio *auio, size_t size)
 {
-    uio_t res;
+    struct uio *res;
     int i;
     user_addr_t iovaddr;
     user_size_t iovsize;
@@ -207,7 +217,6 @@ afsio_darwin_partialcopy(uio_t auio, int size)
 
 vfs_context_t afs_osi_ctxtp;
 int afs_osi_ctxtp_initialized;
-static thread_t vfs_context_owner;
 static proc_t vfs_context_curproc;
 int vfs_context_ref;
 
