@@ -180,26 +180,6 @@ afs_FlushVCache(struct vcache *avc, int *slept)
     /* remove entry from the volume hash table */
     QRemove(&avc->vhashq);
 
-#if defined(AFS_LINUX26_ENV)
-    {
-	struct pagewriter *pw, *store;
-	struct list_head tofree;
-
-	INIT_LIST_HEAD(&tofree);
-	spin_lock(&avc->pagewriter_lock);
-	list_for_each_entry_safe(pw, store, &avc->pagewriters, link) {
-	    list_del(&pw->link);
-	    /* afs_osi_Free may sleep so we need to defer it */
-	    list_add_tail(&pw->link, &tofree);
-	}
-	spin_unlock(&avc->pagewriter_lock);
-	list_for_each_entry_safe(pw, store, &tofree, link) {
-	    list_del(&pw->link);
-	    afs_osi_Free(pw, sizeof(struct pagewriter));
-	}
-    }
-#endif
-
     if (avc->mvid)
 	osi_FreeSmallSpace(avc->mvid);
     avc->mvid = (struct VenusFid *)0;
@@ -225,6 +205,8 @@ afs_FlushVCache(struct vcache *avc, int *slept)
     vn_reinit(AFSTOV(avc));
 #endif
     afs_FreeAllAxs(&(avc->Access));
+    if (!afs_shuttingdown)
+	afs_QueueVCB(avc, slept);
     ObtainWriteLock(&afs_xcbhash, 460);
     afs_DequeueCallback(avc);	/* remove it from queued callbacks list */
     avc->f.states &= ~(CStatd | CUnique);
@@ -234,9 +216,6 @@ afs_FlushVCache(struct vcache *avc, int *slept)
     else
 	osi_dnlc_purgevp(avc);
 
-    if (afs_shuttingdown == AFS_RUNNING)
-	afs_QueueVCB(avc, slept);
-
     /*
      * Next, keep track of which vnodes we've deleted for create's
      * optimistic synchronization algorithm
@@ -396,8 +375,8 @@ FlushAllVCBs(int nconns, struct rx_connection **rxconns,
     for ( i = 0 ; i < nconns ; i++ ) {
 	if (results[i] == 0) {
 	    /* Unchain all of them */
-	    while (conns[i]->srvr->server->cbrs)
-		afs_FreeCBR(conns[i]->srvr->server->cbrs);
+	    while (conns[i]->parent->srvr->server->cbrs)
+		afs_FreeCBR(conns[i]->parent->srvr->server->cbrs);
 	}
     }
     afs_osi_Free(results, nconns * sizeof(afs_int32));
@@ -423,7 +402,7 @@ afs_FlushVCBs(afs_int32 lockit)
     int tcount;
     struct server *tsp;
     int i;
-    struct vrequest *treq = NULL;
+    struct vrequest treq;
     struct afs_conn *tc;
     int safety1, safety2, safety3;
     XSTATS_DECLS;
@@ -431,9 +410,9 @@ afs_FlushVCBs(afs_int32 lockit)
     if (AFS_IS_DISCONNECTED)
 	return ENETDOWN;
 
-    if ((code = afs_CreateReq(&treq, afs_osi_credp)))
+    if ((code = afs_InitReq(&treq, afs_osi_credp)))
 	return code;
-    treq->flags |= O_NONBLOCK;
+    treq.flags |= O_NONBLOCK;
     tfids = afs_osi_Alloc(sizeof(struct AFSFid) * AFS_MAXCBRSCALL);
     osi_Assert(tfids != NULL);
 
@@ -476,8 +455,8 @@ afs_FlushVCBs(afs_int32 lockit)
 		    callBacks[0].CallBackType = CB_EXCLUSIVE;
 		    for (safety3 = 0; safety3 < AFS_MAXHOSTS * 2; safety3++) {
 			tc = afs_ConnByHost(tsp, tsp->cell->fsport,
-					    tsp->cell->cellNum, treq, 0,
-					    SHARED_LOCK, 0, &rxconn);
+					    tsp->cell->cellNum, &treq, 0,
+					    SHARED_LOCK, &rxconn);
 			if (tc) {
 			    XSTATS_START_TIME
 				(AFS_STATS_FS_RPCIDX_GIVEUPCALLBACKS);
@@ -490,7 +469,7 @@ afs_FlushVCBs(afs_int32 lockit)
 			} else
 			    code = -1;
 			if (!afs_Analyze
-			    (tc, rxconn, code, 0, treq,
+			    (tc, rxconn, code, 0, &treq,
 			     AFS_STATS_FS_RPCIDX_GIVEUPCALLBACKS, SHARED_LOCK,
 			     tsp->cell)) {
 			    break;
@@ -535,7 +514,6 @@ afs_FlushVCBs(afs_int32 lockit)
     if (lockit)
 	ReleaseWriteLock(&afs_xvcb);
     afs_osi_Free(tfids, sizeof(struct AFSFid) * AFS_MAXCBRSCALL);
-    afs_DestroyReq(treq);
     return 0;
 }
 
@@ -725,27 +703,21 @@ int
 afs_ShakeLooseVCaches(afs_int32 anumber)
 {
     afs_int32 i, loop;
-    int evicted;
     struct vcache *tvc;
     struct afs_q *tq, *uq;
     int fv_slept, defersleep = 0;
-    int limit;
     afs_int32 target = anumber;
 
-    loop = 0;
-
- retry:
     i = 0;
-    limit = afs_vcount;
+    loop = 0;
     for (tq = VLRU.prev; tq != &VLRU && anumber > 0; tq = uq) {
 	tvc = QTOV(tq);
 	uq = QPrev(tq);
 	if (tvc->f.states & CVFlushed) {
 	    refpanic("CVFlushed on VLRU");
-	} else if (i++ > limit) {
-	    afs_warn("afs_ShakeLooseVCaches: i %d limit %d afs_vcount %d afs_maxvcount %d\n",
-	             (int)i, limit, (int)afs_vcount, (int)afs_maxvcount);
-	    refpanic("Found too many AFS vnodes on VLRU (VLRU cycle?)");
+	    /* In the other path, this was 2 * afs_cacheStats */
+	} else if (!afsd_dynamic_vcaches && i++ > afs_maxvcount) {
+	    refpanic("Exceeded pool of AFS vnodes(VLRU cycle?)");
 	} else if (QNext(uq) != tq) {
 	    refpanic("VLRU inconsistent");
 	} else if (tvc->f.states & CVInit) {
@@ -753,29 +725,21 @@ afs_ShakeLooseVCaches(afs_int32 anumber)
 	}
 
 	fv_slept = 0;
-	evicted = osi_TryEvictVCache(tvc, &fv_slept, defersleep);
-	if (evicted) {
+	if (osi_TryEvictVCache(tvc, &fv_slept, defersleep))
 	    anumber--;
-	}
 
 	if (fv_slept) {
 	    if (loop++ > 100)
 		break;
-	    if (!evicted) {
-		/*
-		 * This vcache was busy and we slept while trying to evict it.
-		 * Move this busy vcache to the head of the VLRU so vcaches
-		 * following this busy vcache can be evicted during the retry.
-		 */
-		QRemove(&tvc->vlruq);
-		QAdd(&VLRU, &tvc->vlruq);
-	    }
-	    goto retry;	/* start over - may have raced. */
+	    uq = VLRU.prev;
+	    i = 0;
+	    continue;	/* start over - may have raced. */
 	}
-	if (uq == &VLRU) {
+	if (tq == uq) {
 	    if (anumber && !defersleep) {
 		defersleep = 1;
-		goto retry;
+		tq = VLRU.prev;
+		continue;
 	    }
 	    break;
 	}
@@ -1028,18 +992,11 @@ afs_FlushActiveVcaches(afs_int32 doflocks)
     struct afs_conn *tc;
     afs_int32 code;
     afs_ucred_t *cred = NULL;
-    struct vrequest *treq = NULL;
+    struct vrequest treq, ureq;
     struct AFSVolSync tsync;
     int didCore;
     XSTATS_DECLS;
     AFS_STATCNT(afs_FlushActiveVcaches);
-
-    code = afs_CreateReq(&treq, afs_osi_credp);
-    if (code) {
-	afs_warn("unable to alloc treq\n");
-	return;
-    }
-
     ObtainReadLock(&afs_xvcache);
     for (i = 0; i < VCSIZE; i++) {
 	for (tvc = afs_vhashT[i]; tvc; tvc = tvc->hnext) {
@@ -1056,14 +1013,10 @@ afs_FlushActiveVcaches(afs_int32 doflocks)
 		ReleaseReadLock(&afs_xvcache);
 		ObtainWriteLock(&tvc->lock, 51);
 		do {
-		    code = afs_InitReq(treq, afs_osi_credp);
-		    if (code) {
-			code = -1;
-			break; /* shutting down: do not try to extend the lock */
-		    }
-		    treq->flags |= O_NONBLOCK;
+		    afs_InitReq(&treq, afs_osi_credp);
+		    treq.flags |= O_NONBLOCK;
 
-		    tc = afs_Conn(&tvc->f.fid, treq, SHARED_LOCK, &rxconn);
+		    tc = afs_Conn(&tvc->f.fid, &treq, SHARED_LOCK, &rxconn);
 		    if (tc) {
 			XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_EXTENDLOCK);
 			RX_AFS_GUNLOCK();
@@ -1076,7 +1029,7 @@ afs_FlushActiveVcaches(afs_int32 doflocks)
 		    } else
 			code = -1;
 		} while (afs_Analyze
-			 (tc, rxconn, code, &tvc->f.fid, treq,
+			 (tc, rxconn, code, &tvc->f.fid, &treq,
 			  AFS_STATS_FS_RPCIDX_EXTENDLOCK, SHARED_LOCK, NULL));
 
 		ReleaseWriteLock(&tvc->lock);
@@ -1113,13 +1066,11 @@ afs_FlushActiveVcaches(afs_int32 doflocks)
 		    /* XXXX Find better place-holder for cred XXXX */
 		    cred = (afs_ucred_t *)tvc->linkData;
 		    tvc->linkData = NULL;	/* XXX */
-		    code = afs_InitReq(treq, cred);
+		    afs_InitReq(&ureq, cred);
 		    afs_Trace2(afs_iclSetp, CM_TRACE_ACTCCORE,
 			       ICL_TYPE_POINTER, tvc, ICL_TYPE_INT32,
 			       tvc->execsOrWriters);
-		    if (!code) {  /* avoid store when shutting down */
-		        code = afs_StoreOnLastReference(tvc, treq);
-		    }
+		    code = afs_StoreOnLastReference(tvc, &ureq);
 		    ReleaseWriteLock(&tvc->lock);
 #ifdef AFS_BOZONLOCK_ENV
 		    afs_BozonUnlock(&tvc->pvnLock, tvc);
@@ -1177,7 +1128,6 @@ afs_FlushActiveVcaches(afs_int32 doflocks)
 	}
     }
     ReleaseReadLock(&afs_xvcache);
-    afs_DestroyReq(treq);
 }
 
 
@@ -1505,6 +1455,9 @@ afs_ProcessFS(struct vcache *avc,
 	      struct AFSFetchStatus *astat, struct vrequest *areq)
 {
     afs_size_t length;
+#ifdef AFS_DARWIN80_ENV
+    int fixup = 0;
+#endif
     AFS_STATCNT(afs_ProcessFS);
 
 #ifdef AFS_64BIT_CLIENT
@@ -1539,16 +1492,32 @@ afs_ProcessFS(struct vcache *avc,
     avc->f.m.Group = astat->Group;
     avc->f.m.LinkCount = astat->LinkCount;
     if (astat->FileType == File) {
+#ifdef AFS_DARWIN80_ENV
+	if (avc->f.m.Type != VREG)
+	    fixup = 1;
+#endif
 	vSetType(avc, VREG);
 	avc->f.m.Mode |= S_IFREG;
     } else if (astat->FileType == Directory) {
+#ifdef AFS_DARWIN80_ENV
+	if (avc->f.m.Type != VDIR)
+	    fixup = 1;
+#endif
 	vSetType(avc, VDIR);
 	avc->f.m.Mode |= S_IFDIR;
     } else if (astat->FileType == SymbolicLink) {
 	if (afs_fakestat_enable && (avc->f.m.Mode & 0111) == 0) {
+#ifdef AFS_DARWIN80_ENV
+	    if (avc->f.m.Type != VDIR)
+		fixup = 1;
+#endif
 	    vSetType(avc, VDIR);
 	    avc->f.m.Mode |= S_IFDIR;
 	} else {
+#ifdef AFS_DARWIN80_ENV
+	    if (avc->f.m.Type != VLNK)
+		fixup = 1;
+#endif
 	    vSetType(avc, VLNK);
 	    avc->f.m.Mode |= S_IFLNK;
 	}
@@ -1556,6 +1525,10 @@ afs_ProcessFS(struct vcache *avc,
 	    avc->mvstat = 1;
 	}
     }
+#ifdef AFS_DARWIN80_ENV
+    if (fixup)
+	printf("found mistyped vnode!\n");
+#endif
     avc->f.anyAccess = astat->AnonymousAccess;
 #ifdef badidea
     if ((astat->CallerAccess & ~astat->AnonymousAccess))
@@ -1615,7 +1588,7 @@ afs_RemoteLookup(struct VenusFid *afid, struct vrequest *areq,
 	tc = afs_Conn(afid, areq, SHARED_LOCK, &rxconn);
 	if (tc) {
 	    if (serverp)
-		*serverp = tc->srvr->server;
+		*serverp = tc->parent->srvr->server;
 	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_XLOOKUP);
 	    RX_AFS_GUNLOCK();
 	    code =
@@ -2373,49 +2346,6 @@ afs_UpdateStatus(struct vcache *avc, struct VenusFid *afid,
     	afs_PutVolume(volp, READ_LOCK);
 }
 
-void
-afs_BadFetchStatus(struct afs_conn *tc)
-{
-    int addr = ntohl(tc->srvr->sa_ip);
-    afs_warn("afs: Invalid AFSFetchStatus from server %u.%u.%u.%u\n",
-             (addr >> 24) & 0xff, (addr >> 16) & 0xff, (addr >> 8) & 0xff,
-             (addr) & 0xff);
-    afs_warn("afs: This suggests the server may be sending bad data that "
-             "can lead to availability issues or data corruption. The "
-             "issue has been avoided for now, but it may not always be "
-             "detectable. Please upgrade the server if possible.\n");
-}
-
-/**
- * Check if a given AFSFetchStatus structure is sane.
- *
- * @param[in] tc The server from which we received the status
- * @param[in] status The status we received
- *
- * @return whether the given structure is valid or not
- *  @retval 0 the structure is fine
- *  @retval nonzero the structure looks like garbage; act as if we received
- *                  the returned error code from the server
- */
-int
-afs_CheckFetchStatus(struct afs_conn *tc, struct AFSFetchStatus *status)
-{
-    if (status->errorCode ||
-        status->InterfaceVersion != 1 ||
-        !(status->FileType > Invalid && status->FileType <= SymbolicLink) ||
-        status->ParentVnode == 0 || status->ParentUnique == 0) {
-
-	afs_warn("afs: FetchStatus ec %u iv %u ft %u pv %u pu %u\n",
-	         (unsigned)status->errorCode, (unsigned)status->InterfaceVersion,
-	         (unsigned)status->FileType, (unsigned)status->ParentVnode,
-	         (unsigned)status->ParentUnique);
-	afs_BadFetchStatus(tc);
-
-	return VBUSY;
-    }
-    return 0;
-}
-
 /*!
  * Must be called with avc write-locked
  * don't absolutely have to invalidate the hint unless the dv has
@@ -2436,7 +2366,7 @@ afs_FetchStatus(struct vcache * avc, struct VenusFid * afid,
 	tc = afs_Conn(afid, areq, SHARED_LOCK, &rxconn);
 	avc->dchint = NULL;	/* invalidate hints */
 	if (tc) {
-	    avc->callback = tc->srvr->server;
+	    avc->callback = tc->parent->srvr->server;
 	    start = osi_Time();
 	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_FETCHSTATUS);
 	    RX_AFS_GUNLOCK();
@@ -2447,10 +2377,6 @@ afs_FetchStatus(struct vcache * avc, struct VenusFid * afid,
 
 	    XSTATS_END_TIME;
 
-	    if (code == 0) {
-		code = afs_CheckFetchStatus(tc, Outsp);
-	    }
-
 	} else
 	    code = -1;
     } while (afs_Analyze
@@ -2652,14 +2578,11 @@ afs_PutVCache(struct vcache *avc)
  *
  * \param avc Pointer to the cache entry to reset
  * \param acred
- * \param skipdnlc  skip the dnlc purge for this vnode
  *
  * \note avc must be write locked on entry
- *
- * \note The caller should purge the dnlc when skipdnlc is set.
  */
 void
-afs_ResetVCache(struct vcache *avc, afs_ucred_t *acred, afs_int32 skipdnlc)
+afs_ResetVCache(struct vcache *avc, afs_ucred_t *acred)
 {
     ObtainWriteLock(&afs_xcbhash, 456);
     afs_DequeueCallback(avc);
@@ -2667,9 +2590,7 @@ afs_ResetVCache(struct vcache *avc, afs_ucred_t *acred, afs_int32 skipdnlc)
     ReleaseWriteLock(&afs_xcbhash);
     /* now find the disk cache entries */
     afs_TryToSmush(avc, acred, 1);
-    if (!skipdnlc) {
-	osi_dnlc_purgedp(avc);
-    }
+    osi_dnlc_purgedp(avc);
     if (avc->linkData && !(avc->f.states & CCore)) {
 	afs_osi_Free(avc->linkData, strlen(avc->linkData) + 1);
 	avc->linkData = NULL;
