@@ -52,6 +52,9 @@
 # include <sys/timeout.h>
 
 #elif defined(AFS_NBSD40_ENV)
+# ifdef AFS_NBSD50_ENV
+#  include <sys/simplelock.h>
+# endif
 # include <sys/errno.h>
 # include <sys/types.h>
 # include <sys/mount.h> /* may define MOUNT_AFS */
@@ -82,21 +85,14 @@
 # include <sys/socket.h>
 # include <sys/socketvar.h>
 # include <sys/dirent.h>
-# include <sys/user.h>
 # include <sys/kauth.h>
 # include <sys/uio.h>
 # include <sys/buf.h>
 # include <sys/stat.h>
 # include <sys/file.h>
 # include <sys/namei.h>
-# include <sys/socket.h>
-# include <sys/socketvar.h>
-# include <sys/dirent.h>
-# include <sys/user.h>
-# include <sys/kauth.h>
 # include <sys/uio.h>
 # include <sys/buf.h>
-# include <sys/stat.h>
 # include <sys/file.h>
 # include <sys/namei.h>
 # include <sys/vnode.h>
@@ -106,9 +102,7 @@
 # include <ufs/ufs/extattr.h>
 # include <ufs/ufs/ufsmount.h>
 # ifndef MLEN
-#  if 0
-#   include <sys/mbuf.h>
-#  endif /* 0 */
+#  include <sys/mbuf.h>
 #  include <net/if.h>
 # endif /* !MLEN */
 # include <sys/protosw.h>
@@ -151,23 +145,15 @@ struct coda_inode_info {
 struct xfs_inode_info {
 };
 # endif
-# if defined(HAVE_LINUX_UACCESS_H)
-#  include <linux/uaccess.h>
-# else
-#  include <asm/uaccess.h>
-# endif
+# include <asm/uaccess.h>
 # include <linux/list.h>
 # include <linux/dcache.h>
 # include <linux/mount.h>
 # include <linux/fs.h>
 # include <linux/quota.h>
 # include <linux/sched.h>
-# if defined(HAVE_LINUX_SCHED_SIGNAL_H)
-#  include <linux/sched/signal.h>
-# endif
 # include <linux/mm.h>
 # include <linux/slab.h>
-# include <linux/proc_fs.h>
 # include <linux/string.h>
 # if defined(HAVE_LINUX_SEMAPHORE_H)
 #  include <linux/semaphore.h>
@@ -215,12 +201,6 @@ typedef unsigned short etap_event_t;
 # include "h/systm.h"
 # include "h/time.h"
 
-# ifdef AFS_FBSD_ENV
-#  include <sys/socket.h>
-#  include <net/if.h>
-#  include <net/if_var.h>
-# endif
-
 # ifdef	AFS_AIX_ENV
 #  ifdef AFS_AIX41_ENV
 #   include "sys/statfs.h"
@@ -283,12 +263,9 @@ typedef unsigned short etap_event_t;
 
 # ifdef	AFS_SUN5_ENV
 #  include <sys/cmn_err.h>	/* for kernel printf() prototype */
-# endif
-
-# if	defined(AFS_SUN56_ENV)
-#  include "h/vfs.h"		/* stops SUN56 socketvar.h warnings */
-#  include "h/stropts.h"		/* stops SUN56 socketvar.h warnings */
-#  include "h/stream.h"		/* stops SUN56 socketvar.h errors */
+#  include "h/vfs.h"		/* stops SUN5 socketvar.h warnings */
+#  include "h/stropts.h"		/* stops SUN5 socketvar.h warnings */
+#  include "h/stream.h"		/* stops SUN5 socketvar.h errors */
 # endif
 
 # ifdef AFS_SUN510_ENV
@@ -301,7 +278,7 @@ typedef unsigned short etap_event_t;
 # include "h/protosw.h"
 
 # if defined(AFS_FBSD_ENV)
-#  include "sys/limits.h"
+#  include "limits.h"
 # endif
 
 # ifdef AFS_HPUX_ENV
