@@ -5,6 +5,8 @@
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
+ *
+ * Portions Copyright (c) 2003 Apple Computer, Inc.
  */
 
 /*
@@ -14,30 +16,33 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/libadmin/samples/rxstat_query_peer.c,v 1.4 2001/07/12 19:58:45 shadow Exp $");
+#include <roken.h>
 
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <pthread.h>
 #endif
+
+#include <rx/rx.h>
+#include <rx/rxstat.h>
 #include <afs/afs_Admin.h>
 #include <afs/afs_clientAdmin.h>
 #include <afs/afs_utilAdmin.h>
 
-extern int RXSTATS_QueryPeerRPCStats();
+#ifdef AFS_DARWIN_ENV
+pthread_mutex_t des_init_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t des_random_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t rxkad_random_mutex = PTHREAD_MUTEX_INITIALIZER;
+#endif /* AFS_DARWIN_ENV */
 
-void Usage()
+void
+Usage(void)
 {
-    fprintf(stderr,
-	    "Usage: rxstat_query_peer <host> <port>\n");
+    fprintf(stderr, "Usage: rxstat_query_peer <host> <port>\n");
     exit(1);
 }
 
-void ParseArgs(
-    int argc,
-    char *argv[],
-    char **srvrName,
-    long *srvrPort)
+void
+ParseArgs(int argc, char *argv[], char **srvrName, long *srvrPort)
 {
     char **argp = argv;
 
@@ -53,7 +58,8 @@ void ParseArgs(
 	Usage();
 }
 
-int main(int argc, char *argv[])
+int
+main(int argc, char *argv[])
 {
     int rc;
     afs_status_t st = 0;
@@ -77,7 +83,8 @@ int main(int argc, char *argv[])
 	exit(1);
     }
 
-    rc = afsclient_RPCStatOpenPort(cellHandle, srvrName, srvrPort, &conn, &st);
+    rc = afsclient_RPCStatOpenPort(cellHandle, srvrName, srvrPort, &conn,
+				   &st);
     if (!rc) {
 	fprintf(stderr, "afsclient_RPCStatOpenPort, status %d\n", st);
 	exit(1);
@@ -103,14 +110,14 @@ int main(int argc, char *argv[])
 
     printf("\n");
     printf("Peer RPC stats are ");
-    switch(state) {
-      case AFS_RPC_STATS_DISABLED:
+    switch (state) {
+    case AFS_RPC_STATS_DISABLED:
 	printf("disabled\n");
 	break;
-      case AFS_RPC_STATS_ENABLED:
+    case AFS_RPC_STATS_ENABLED:
 	printf("enabled\n");
 	break;
-      default:
+    default:
 	printf("INVALID\n");
 	break;
     }
