@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,27 +9,19 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-RCSID
-    ("$Header: /cvs/openafs/src/procmgmt/procmgmt_nt.c,v 1.7.4.2 2007/06/29 23:52:54 jaltman Exp $");
-
 #include <afs/stds.h>
 
-#include <stddef.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <errno.h>
-#include <string.h>
+#include "procmgmt.h" /* Must be before roken */
+
+#include <roken.h>
+
 #include <windows.h>
 #include <pthread.h>
 #include <afs/errmap_nt.h>
 #include <afs/secutil_nt.h>
 
-#include "procmgmt.h"
 #include "pmgtprivate.h"
 
-
-
 /* Signal disposition table and associated definitions and locks */
 
 typedef struct {
@@ -611,7 +603,7 @@ StringArrayToMultiString(char *strArray[])
 		    bufp += strLen + 1;
 		}
 	    }
-	    bufp = '\0';	/* terminate multistring */
+	    *bufp = '\0';	/* terminate multistring */
 	}
     }
 
@@ -884,6 +876,7 @@ pmgt_ProcessSpawnVEB(const char *spath, char *sargv[], char *senvp[],
     DWORD monitorId, createFlags;
     BOOL passingBuffer = (sdatap != NULL && sdatalen > 0);
     BOOL fsuccess;
+    int lasterror;
 
     /* verify arguments */
     if (!spath || !sargv) {
@@ -984,13 +977,14 @@ pmgt_ProcessSpawnVEB(const char *spath, char *sargv[], char *senvp[],
 			     &startInfo,	/* startup info block */
 			     &procInfo);
 
+    lasterror = GetLastError();
     free(pathbuf);
     free(argbuf);
     free(envbuf);
 
     if (!fsuccess) {
 	/* failed to spawn process */
-	errno = nterr_nt2unix(GetLastError(), ENOENT);
+	errno = nterr_nt2unix(lasterror, ENOENT);
 
 	(void)pthread_mutex_lock(&procTableLock);
 	procTable[tidx].p_reserved = FALSE;	/* mark entry as not reserved */
