@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -21,38 +21,24 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_lwp.c,v 1.19.4.3 2008/05/20 21:59:02 shadow Exp $");
+#include <roken.h>
 
-# include <sys/types.h>		/* fd_set on older platforms */
-# include <errno.h>
-# include <signal.h>
-#ifdef AFS_NT40_ENV
-# include <winsock2.h>
-#else
-# include <unistd.h>		/* select() prototype */
-# include <sys/time.h>		/* struct timeval, select() prototype */
-# ifndef FD_SET
-#  include <sys/select.h>	/* fd_set on newer platforms */
-# endif
-# include <sys/socket.h>
+#ifdef HAVE_SYS_FILE_H
 # include <sys/file.h>
-# include <netdb.h>
-# include <sys/stat.h>
-# include <netinet/in.h>
-# include <net/if.h>
-# include <sys/ioctl.h>
-# include <sys/time.h>
 #endif
-# include "rx.h"
-# include "rx_globals.h"
-# include <lwp.h>
+
+#include <lwp.h>
+
+#include "rx.h"
+#include "rx_atomic.h"
+#include "rx_globals.h"
+#include "rx_stats.h"
+#ifdef AFS_NT40_ENV
+#include "rx_xmit_nt.h"
+#endif
 
 #define MAXTHREADNAMELENGTH 64
 
-extern int (*registerProgram) ();
-extern int (*swapNameProgram) ();
-
 int debugSelectFailure;		/* # of times select failed */
 
 /*
@@ -138,7 +124,7 @@ rxi_StartListener(void)
     /* Priority of listener should be high, so it can keep conns alive */
 #define	RX_LIST_STACK	24000
     LWP_CreateProcess(rx_ListenerProc, RX_LIST_STACK, LWP_MAX_PRIORITY,
-		      (void *)0, "rx_Listener", &rx_listenerPid);
+		      NULL, "rx_Listener", &rx_listenerPid);
     if (registerProgram)
 	(*registerProgram) (rx_listenerPid, "listener");
 }
@@ -165,7 +151,7 @@ rxi_ListenerProc(fd_set * rfds, int *tnop, struct rx_call **newcallp)
 {
     afs_uint32 host;
     u_short port;
-    register struct rx_packet *p = (struct rx_packet *)0;
+    struct rx_packet *p = (struct rx_packet *)0;
     osi_socket socket;
     struct clock cv;
     afs_int32 nextPollTime;	/* time to next poll FD before sleeping */
@@ -183,14 +169,16 @@ rxi_ListenerProc(fd_set * rfds, int *tnop, struct rx_call **newcallp)
     nextPollTime = 0;
     code = LWP_CurrentProcess(&pid);
     if (code) {
-	fprintf(stderr, "rxi_Listener: Can't get my pid.\n");
-	exit(1);
+	osi_Panic("rxi_Listener: Can't get my pid.\n");
     }
     rx_listenerPid = pid;
     if (swapNameProgram)
 	(*swapNameProgram) (pid, "listener", &name[0]);
 
     for (;;) {
+        /* See if a check for additional packets was issued */
+        rx_CheckPackets();
+
 	/* Grab a new packet only if necessary (otherwise re-use the old one) */
 	if (p) {
 	    rxi_RestoreDataBufs(p);
@@ -214,7 +202,8 @@ rxi_ListenerProc(fd_set * rfds, int *tnop, struct rx_call **newcallp)
 	    tv.tv_usec = cv.usec;
 	    tvp = &tv;
 	}
-	rx_stats.selects++;
+	if (rx_stats_active)
+	    rx_atomic_inc(&rx_stats.selects);
 
 	*rfds = rx_selectMask;
 
@@ -283,7 +272,7 @@ rxi_ListenerProc(fd_set * rfds, int *tnop, struct rx_call **newcallp)
 			    rxi_FreePacket(p);
 			}
 			if (swapNameProgram) {
-			    (*swapNameProgram) (rx_listenerPid, &name, 0);
+			    (*swapNameProgram) (rx_listenerPid, name, 0);
 			    rx_listenerPid = 0;
 			}
 			return;
@@ -303,7 +292,7 @@ rxi_ListenerProc(fd_set * rfds, int *tnop, struct rx_call **newcallp)
 			    rxi_FreePacket(p);
 			}
 			if (swapNameProgram) {
-			    (*swapNameProgram) (rx_listenerPid, &name, 0);
+			    (*swapNameProgram) (rx_listenerPid, name, 0);
 			    rx_listenerPid = 0;
 			}
 			return;
@@ -336,11 +325,11 @@ rx_ListenerProc(void *dummy)
 	newcall = NULL;
 	threadID = -1;
 	rxi_ListenerProc(rfds, &threadID, &newcall);
-	/* assert(threadID != -1); */
-	/* assert(newcall != NULL); */
+	/* osi_Assert(threadID != -1); */
+	/* osi_Assert(newcall != NULL); */
 	sock = OSI_NULLSOCKET;
 	rxi_ServerProc(threadID, newcall, &sock);
-	/* assert(sock != OSI_NULLSOCKET); */
+	/* osi_Assert(sock != OSI_NULLSOCKET); */
     }
     /* not reached */
     return NULL;
@@ -370,11 +359,11 @@ rx_ServerProc(void * unused)
     while (1) {
 	sock = OSI_NULLSOCKET;
 	rxi_ServerProc(threadID, newcall, &sock);
-	/* assert(sock != OSI_NULLSOCKET); */
+	/* osi_Assert(sock != OSI_NULLSOCKET); */
 	newcall = NULL;
 	rxi_ListenerProc(rfds, &threadID, &newcall);
-	/* assert(threadID != -1); */
-	/* assert(newcall != NULL); */
+	/* osi_Assert(threadID != -1); */
+	/* osi_Assert(newcall != NULL); */
     }
     /* not reached */
     return NULL;
@@ -393,19 +382,11 @@ rxi_Listen(osi_socket sock)
      * Put the socket into non-blocking mode so that rx_Listener
      * can do a polling read before entering select
      */
-#ifndef AFS_DJGPP_ENV
     if (fcntl(sock, F_SETFL, FNDELAY) == -1) {
 	perror("fcntl");
 	(osi_Msg "rxi_Listen: unable to set non-blocking mode on socket\n");
 	return -1;
     }
-#else
-    if (__djgpp_set_socket_blocking_mode(sock, 1) < 0) {
-	perror("__djgpp_set_socket_blocking_mode");
-	(osi_Msg "rxi_Listen: unable to set non-blocking mode on socket\n");
-	return -1;
-    }
-#endif /* AFS_DJGPP_ENV */
 
     if (sock > FD_SETSIZE - 1) {
 	(osi_Msg "rxi_Listen: socket descriptor > (FD_SETSIZE-1) = %d\n",
@@ -445,7 +426,8 @@ rxi_Sendmsg(osi_socket socket, struct msghdr *msg_p, int flags)
     fd_set *sfds = (fd_set *) 0;
     while (sendmsg(socket, msg_p, flags) == -1) {
 	int err;
-	rx_stats.sendSelects++;
+	if (rx_stats_active)
+	    rx_atomic_inc(&rx_stats.sendSelects);
 
 	if (!sfds) {
 	    if (!(sfds = IOMGR_AllocFDSet())) {
@@ -473,11 +455,26 @@ rxi_Sendmsg(osi_socket socket, struct msghdr *msg_p, int flags)
 	{
 	    (osi_Msg "rx failed to send packet: ");
 	    perror("rx_sendmsg");
+#ifndef AFS_NT40_ENV
+            if (errno > 0)
+              return -errno;
+#else
+            if (WSAGetLastError() > 0)
+              return -WSAGetLastError();
+#endif
 	    return -1;
 	}
-	while ((err = select(socket + 1, 0, sfds, 0, 0)) != 1) {
+	while ((err = select(
+#ifdef AFS_NT40_ENV
+                             0,
+#else
+                             socket + 1,
+#endif
+                             0, sfds, 0, 0)) != 1) {
 	    if (err >= 0 || errno != EINTR)
 		osi_Panic("rxi_sendmsg: select error %d.%d", err, errno);
+	    FD_ZERO(sfds);
+	    FD_SET(socket, sfds);
 	}
     }
     if (sfds)
