@@ -48,6 +48,8 @@
 #include "rx_atomic.h"
 #include "rx_globals.h"
 #include "rx_stats.h"
+#include "rx_packet.h"
+
 #ifdef AFS_PTHREAD_ENV
 
 /*
@@ -233,7 +235,7 @@ osi_Panic(char *msg, ...)
     va_end(ap);
     fflush(stderr);
     fflush(stdout);
-    afs_abort();
+    opr_abort();
 }
 
 /*
@@ -689,7 +691,6 @@ rxi_InitPeerParams(struct rx_peer *pp)
 
     pp->ifMTU = 0;
     rx_rto_setPeerTimeoutSecs(pp, 2);
-    pp->rateFlag = 2;		/* start timing after two full packets */
     /* I don't initialize these, because I presume they are bzero'd...
      * pp->burstSize pp->burst pp->burstWait.sec pp->burstWait.usec
      */
@@ -715,7 +716,6 @@ rxi_InitPeerParams(struct rx_peer *pp)
 	pp->ifMTU = MIN(rx_MyMaxSendSize, RX_REMOTE_PACKET_SIZE);
     }
 #else /* ADAPT_MTU */
-    pp->rateFlag = 2;		/* start timing after two full packets */
     rx_rto_setPeerTimeoutSecs(pp, 2);
     pp->ifMTU = MIN(rx_MyMaxSendSize, OLD_MAX_PACKET_SIZE);
 #endif /* ADAPT_MTU */
@@ -766,10 +766,15 @@ rx_SetNoJumbo(void)
 
 /* Override max MTU.  If rx_SetNoJumbo is called, it must be
    called before calling rx_SetMaxMTU since SetNoJumbo clobbers rx_maxReceiveSize */
-void
+int
 rx_SetMaxMTU(int mtu)
 {
+    if (mtu < RX_MIN_PACKET_SIZE || mtu > RX_MAX_PACKET_DATA_SIZE)
+	return EINVAL;
+
     rx_MyMaxSendSize = rx_maxReceiveSizeUser = rx_maxReceiveSize = mtu;
+
+    return 0;
 }
 
 #if defined(ADAPT_PMTU)
