@@ -10,7 +10,6 @@
 #include "afs/param.h"
 #include <afsconfig.h>
 
-RCSID("$Header: /cvs/openafs/src/rx/test/kctest.c,v 1.4 2001/08/08 00:04:04 shadow Exp $");
 
 #include <sys/types.h>
 #include <netinet/in.h>
@@ -32,9 +31,11 @@ static short secLevel = 0;
 static short stats = 0;
 
 #if RX_VAB_EXISTS
-static MakeVTest(akey, aticket, asession)
-struct rxvab_EncryptionKey *akey, *asession;
-struct rxvab_Ticket *aticket; {
+static
+MakeVTest(akey, aticket, asession)
+     struct rxvab_EncryptionKey *akey, *asession;
+     struct rxvab_Ticket *aticket;
+{
     aticket->ViceId = htonl(71);
     memcpy(&aticket->HandShakeKey, "testkeyx", 8);
     memcpy(asession, "testkeyx", 8);
@@ -45,49 +46,52 @@ struct rxvab_Ticket *aticket; {
 #define MakeVTest(a,b,c) (printf ("rx_vab support removed\n"), exit (-1))
 #endif
 
-void SigInt(int ignore) {
+void
+SigInt(int ignore)
+{
     if (rx_debugFile) {
 	rx_PrintStats(rx_debugFile);
 	fflush(rx_debugFile);
     }
-    if (stats) rx_PrintStats(stdout);
+    if (stats)
+	rx_PrintStats(stdout);
     rx_Finalize();
     exit(1);
 }
 
-static ParseCmd(argc, argv)
-int argc;
-char **argv; {
-    register int i;
-    register struct hostent *th;
-    for(i=1;i<argc;i++) {
-	if (!strcmp(argv[i],"-port")) {
-	    port = atoi(argv[i+1]);
+static
+ParseCmd(argc, argv)
+     int argc;
+     char **argv;
+{
+    int i;
+    struct hostent *th;
+    for (i = 1; i < argc; i++) {
+	if (!strcmp(argv[i], "-port")) {
+	    port = atoi(argv[i + 1]);
 	    i++;
-	}
-	else if (!strcmp(argv[i],"-host")) {
-	    th = gethostbyname(argv[i+1]);
+	} else if (!strcmp(argv[i], "-host")) {
+	    th = gethostbyname(argv[i + 1]);
 	    if (!th) {
-		printf("could not find host '%s' in host table\n", argv[i+1]);
+		printf("could not find host '%s' in host table\n",
+		       argv[i + 1]);
 		return -1;
 	    }
 	    memcpy(&host, th->h_addr, sizeof(long));
 	    i++;
-	}
-	else if (!strcmp(argv[i],"-count")) {
-	    count = atoi(argv[i+1]);
+	} else if (!strcmp(argv[i], "-count")) {
+	    count = atoi(argv[i + 1]);
 	    i++;
-	}
-	else if (!strcmp(argv[i], "-security")) {
-	    secLevel = atoi(argv[i+1]);
+	} else if (!strcmp(argv[i], "-security")) {
+	    secLevel = atoi(argv[i + 1]);
 	    i++;
-	}
-	else if (!strcmp(argv[i],"-log")) {
+	} else if (!strcmp(argv[i], "-log")) {
 	    rx_debugFile = fopen("kctest.log", "w");
-	    if (rx_debugFile == NULL) printf("Couldn't open rx_stest.db");
+	    if (rx_debugFile == NULL)
+		printf("Couldn't open rx_stest.db");
 	    signal(SIGINT, SigInt);
-	}
-	else if (!strcmp(argv[i], "-stats")) stats = 1;
+	} else if (!strcmp(argv[i], "-stats"))
+	    stats = 1;
 	else {
 	    printf("unrecognized switch '%s'\n", argv[i]);
 	    return -1;
@@ -96,18 +100,20 @@ char **argv; {
     return 0;
 }
 
-nowms () {
+nowms()
+{
     struct timeval tv;
     long temp;
 
-    gettimeofday(&tv, 0);
-    temp = ((tv.tv_sec&0xffff)*1000)+(tv.tv_usec/1000);
+    gettimeofday(&tv, NULL);
+    temp = ((tv.tv_sec & 0xffff) * 1000) + (tv.tv_usec / 1000);
     return temp;
 }
 
 main(argc, argv)
-int argc;
-char **argv; {
+     int argc;
+     char **argv;
+{
     struct rx_securityClass *so;
     struct rx_connection *tconn;
     struct rx_call *tcall;
@@ -130,18 +136,18 @@ char **argv; {
     if (secLevel == 0)
 	so = rxnull_NewClientSecurityObject();
     else if (secLevel == 1) {
-	MakeVTest((struct rxvab_EncryptionKey *)"applexxx", &ticket, &session);
+	MakeVTest((struct rxvab_EncryptionKey *)"applexxx", &ticket,
+		  &session);
 #if RX_VAB_EXISTS
 	so = rxvab_NewClientSecurityObject(&session, &ticket, 0);
 #endif
-    }
-    else if (secLevel == 2) {
-	MakeVTest((struct rxvab_EncryptionKey *)"applexxx", &ticket, &session);
+    } else if (secLevel == 2) {
+	MakeVTest((struct rxvab_EncryptionKey *)"applexxx", &ticket,
+		  &session);
 #if RX_VAB_EXISTS
 	so = rxvab_NewClientSecurityObject(&session, &ticket, 1);
 #endif
-    }
-    else {
+    } else {
 	printf("bad security index\n");
 	exit(1);
     }
@@ -153,7 +159,7 @@ char **argv; {
     printf("conn is %x\n", tconn);
 
     startms = nowms();
-    for(i=0;i<count;i++) {
+    for (i = 0; i < count; i++) {
 	tcall = rx_NewCall(tconn);
 	/* fill in data */
 	xdrrx_create(&xdr, tcall, XDR_ENCODE);
@@ -161,14 +167,16 @@ char **argv; {
 	xdr_long(&xdr, &temp);
 	xdr.x_op = XDR_DECODE;
 	xdr_long(&xdr, &temp);
-	if (temp != 1989) printf("wrong value returned (%d)\n", temp);
+	if (temp != 1989)
+	    printf("wrong value returned (%d)\n", temp);
 	rx_EndCall(tcall, 0);
     }
     endms = nowms();
-    printf("That was %d ms per call.\n", (endms-startms)/count);
+    printf("That was %d ms per call.\n", (endms - startms) / count);
     printf("Done.\n");
 #ifdef RXDEBUG
-    if (stats) rx_PrintStats(stdout);
+    if (stats)
+	rx_PrintStats(stdout);
 #endif
     SigInt(0);
 }
