@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,27 +9,15 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
-#ifdef AFS_NT40_ENV
-#include <fcntl.h>
-#include <winsock2.h>
-#else
-#include <sys/types.h>
-#include <sys/file.h>
-#include <netdb.h>
-#include <netinet/in.h>
-#endif /* AFS_NT40_ENV */
-#include <sys/stat.h>
+
+#include <roken.h>
+
 #ifdef AFS_AIX_ENV
-#include <sys/statfs.h>
+# include <sys/statfs.h>
 #endif
 
-#include <string.h>
-
 #include <afs/dirpath.h>
-#include <errno.h>
 #include <lock.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
@@ -46,9 +34,9 @@
  */
 afs_int32
 ugen_ClientInit(int noAuthFlag, const char *confDir, char *cellName, afs_int32 sauth,
-	       struct ubik_client **uclientp, 
+	       struct ubik_client **uclientp,
 	       int (*secproc) (struct rx_securityClass *, afs_int32),
-	       char *funcName, afs_int32 gen_rxkad_level, 
+	       char *funcName, afs_int32 gen_rxkad_level,
 	       afs_int32 maxservers, char *serviceid, afs_int32 deadtime,
 	       afs_uint32 server, afs_uint32 port, afs_int32 usrvid)
 {
@@ -59,7 +47,6 @@ ugen_ClientInit(int noAuthFlag, const char *confDir, char *cellName, afs_int32 s
     struct rx_securityClass *sc;
     /* This must change if VLDB_MAXSERVERS becomes larger than MAXSERVERS */
     static struct rx_connection *serverconns[MAXSERVERS];
-    const char *confdir;
 
     code = rx_Init(0);
     if (code) {
@@ -71,16 +58,21 @@ ugen_ClientInit(int noAuthFlag, const char *confDir, char *cellName, afs_int32 s
     secFlags = AFSCONF_SECOPTS_FALLBACK_NULL;
     if (sauth) {
 	secFlags |= AFSCONF_SECOPTS_LOCALAUTH;
-	confdir = AFSDIR_SERVER_ETC_DIRPATH;
+	confDir = AFSDIR_SERVER_ETC_DIRPATH;
     } else {
-	confdir = AFSDIR_CLIENT_ETC_DIRPATH;
+	if (confDir == NULL)
+	    confDir = AFSDIR_CLIENT_ETC_DIRPATH;
     }
 
-    tdir = afsconf_Open(confdir);
+    if (noAuthFlag) {
+	secFlags |= AFSCONF_SECOPTS_NOAUTH;
+    }
+
+    tdir = afsconf_Open(confDir);
     if (!tdir) {
 	fprintf(stderr,
 		"%s: Could not process files in configuration directory (%s).\n",
-		funcName, confdir);
+		funcName, confDir);
 	return -1;
     }
 
@@ -91,7 +83,7 @@ ugen_ClientInit(int noAuthFlag, const char *confDir, char *cellName, afs_int32 s
     if (code) {
 	afsconf_Close(tdir);
 	fprintf(stderr, "%s: can't find cell %s's hosts in %s/%s\n",
-		funcName, cellName, confdir, AFSDIR_CELLSERVDB_FILE);
+		funcName, cellName, confDir, AFSDIR_CELLSERVDB_FILE);
 	return -1;
     }
     code = afsconf_PickClientSecObj(tdir, secFlags, &info, cellName, &sc,
@@ -100,7 +92,7 @@ ugen_ClientInit(int noAuthFlag, const char *confDir, char *cellName, afs_int32 s
 	fprintf(stderr, "%s: can't create client security object", funcName);
 	return -1;
     }
-    if (scIndex == RX_SECIDX_NULL) {
+    if (scIndex == RX_SECIDX_NULL && !noAuthFlag) {
 	fprintf(stderr,
 		"%s: Could not get afs tokens, running unauthenticated.\n",
 		funcName);
@@ -121,6 +113,8 @@ ugen_ClientInit(int noAuthFlag, const char *confDir, char *cellName, afs_int32 s
 	    return -1;
 	}
 	for (i = 0; i < info.numServers; i++) {
+	    if (!info.hostAddr[i].sin_port && port)
+		info.hostAddr[i].sin_port = port;
 	    serverconns[i] =
 		rx_NewConnection(info.hostAddr[i].sin_addr.s_addr,
 				 info.hostAddr[i].sin_port, usrvid,
