@@ -18,14 +18,31 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/vfsck/main.c,v 1.10 2005/10/03 02:44:56 shadow Exp $");
+#ifdef AFS_HPUX_ENV
+/* We need the old directory type headers (included below), so don't include
+ * the normal dirent.h, or it will conflict. */
+# undef HAVE_DIRENT_H
+# include <sys/inode.h>
+# define	LONGFILENAMES	1
+# include <sys/sysmacros.h>
+# include <sys/ino.h>
+# define	DIRSIZ_MACRO
+# ifdef HAVE_USR_OLD_USR_INCLUDE_NDIR_H
+#  include </usr/old/usr/include/ndir.h>
+# else
+#  include <ndir.h>
+# endif
+#endif
+
+#include <roken.h>
+
+#include <ctype.h>
+
+#ifdef HAVE_SYS_FILE_H
+#include <sys/file.h>
+#endif
 
 #define VICE			/* allow us to put our changes in at will */
-#include <stdio.h>
-
-#include <sys/param.h>
-#include <sys/time.h>
 
 #ifdef AFS_SUN_ENV
 #define KERNEL
@@ -35,8 +52,6 @@ RCSID
 #undef KERNEL
 #endif
 
-#include <sys/file.h>
-
 #ifdef	AFS_OSF_ENV
 #include <sys/vnode.h>
 #include <sys/mount.h>
@@ -47,7 +62,6 @@ RCSID
 #define VFS
 #include <sys/vnode.h>
 #ifdef	  AFS_SUN5_ENV
-#include <unistd.h>
 #include <sys/fs/ufs_inode.h>
 #include <sys/fs/ufs_fs.h>
 #define _KERNEL
@@ -64,18 +78,7 @@ RCSID
 #else /* AFS_VFSINCL_ENV */
 
 #include <sys/inode.h>
-#ifdef	AFS_HPUX_ENV
-#include <ctype.h>
-#define	LONGFILENAMES	1
-#include <sys/sysmacros.h>
-#include <sys/ino.h>
-#define	DIRSIZ_MACRO
-#ifdef HAVE_USR_OLD_USR_INCLUDE_NDIR_H
-#include </usr/old/usr/include/ndir.h>
-#else
-#include <ndir.h>
-#endif
-#else
+#ifndef	AFS_HPUX_ENV
 #define KERNEL
 #include <sys/dir.h>
 #undef KERNEL
@@ -84,14 +87,7 @@ RCSID
 #endif /* AFS_VFSINCL_ENV */
 #endif /* AFS_OSF_ENV */
 
-#include <sys/stat.h>
 #include <sys/wait.h>
-#ifdef	AFS_SUN5_ENV
-#include <string.h>
-#else
-#include <strings.h>
-#endif
-#include <ctype.h>
 #ifdef	XAFS_SUN_ENV
 #include <mntent.h>
 #else
@@ -105,10 +101,9 @@ RCSID
 #endif
 #endif
 #include "fsck.h"
-#include <errno.h>
 #include <sys/signal.h>
 
-char *rawname(), *unrawname(), *blockcheck(), *malloc();
+char *rawname(), *unrawname(), *blockcheck();
 void catch(), catchquit(), voidquit();
 static int tryForce;
 int returntosingle;
@@ -151,10 +146,10 @@ int mnt_passno = 0;
 #include "AFS_component_version_number.c"
 
 #ifdef	AFS_HPUX_ENV
-int ge_danger = 0;		/* on when fsck is not able to fix the dirty file 
+int ge_danger = 0;		/* on when fsck is not able to fix the dirty file
 				 * system within single run. Problems like dup table
 				 * overflow, maxdup is exceeding MAXDUP.. etc. could
-				 * potentailly prevent fsck from doing a complete 
+				 * potentailly prevent fsck from doing a complete
 				 * repair. This is found in a GE hotsite. */
 #endif
 
@@ -165,8 +160,8 @@ main(argc, argv)
     struct fstab *fsp;
     int pid, passno, sumstatus;
     char *name;
-    register struct disk *dk, *nextdisk;
-    register struct part *pt;
+    struct disk *dk, *nextdisk;
+    struct part *pt;
     extern char *AFSVersion;	/* generated version */
 #ifdef	AFS_SUN5_ENV
     int other_than_ufs = 0;
@@ -388,7 +383,7 @@ main(argc, argv)
 	    hotroot = 0;
 #ifdef	AFS_SUN5_ENV
 	    if (wflag && !writable(*argv)) {
-		(void)fprintf(stderr, "not writeable '%s'\n", *argv);
+		(void)fprintf(stderr, "not writable '%s'\n", *argv);
 		argv++;
 	    } else
 #endif
@@ -547,8 +542,8 @@ struct disk *
 finddisk(name)
      char *name;
 {
-    register struct disk *dk, **dkp;
-    register char *p;
+    struct disk *dk, **dkp;
+    char *p;
     int len;
 
     for (p = name + strlen(name) - 1; p >= name; --p)
@@ -581,7 +576,7 @@ addpart(name, fsname)
      char *name, *fsname;
 {
     struct disk *dk = finddisk(name);
-    register struct part *pt, **ppt = &dk->part;
+    struct part *pt, **ppt = &dk->part;
 
     for (pt = dk->part; pt; ppt = &pt->next, pt = pt->next)
 	if (strcmp(pt->name, name) == 0) {
@@ -601,7 +596,7 @@ addpart(name, fsname)
 }
 
 startdisk(dk)
-     register struct disk *dk;
+     struct disk *dk;
 {
 
     nrun++;
@@ -639,7 +634,7 @@ checkfilesys(filesys, parname)
     isconvert = 0;
 #endif
 #ifdef	AFS_HPUX_ENV
-    ge_danger = 0;		/* set to 1 by any table overflow or more 
+    ge_danger = 0;		/* set to 1 by any table overflow or more
 				 * dup/bad blocks than expected */
 
     fixed = 1;			/* set to 0 by any 'no' reply */
@@ -826,7 +821,7 @@ checkfilesys(filesys, parname)
 		    sblock.fs_clean = FS_CLEAN;
 		/*
 		 *  Fix fs_clean if there were no 'no' replies.
-		 *  This is done for both the s300 and s800.  The s800 root will be 
+		 *  This is done for both the s300 and s800.  The s800 root will be
 		 *  guaranteed clean as of 7.0.
 		 */
 		if (fixed && (sblock.fs_clean != FS_OK)) {
@@ -1160,7 +1155,7 @@ check_sanity(filename)
     /*
      * We mount the ufs root file system read-only first.  After fsck
      * runs, we remount the root as read-write.  Therefore, we no longer
-     * check for different values for fs_state between the root file 
+     * check for different values for fs_state between the root file
      * system and the rest of file systems.
      */
     if (!((sblock.fs_state + (time_t) sblock.fs_time == FSOKAY)
@@ -1236,7 +1231,7 @@ check_sanity(filename)
     /*
      * We mount the ufs root file system read-only first.  After fsck
      * runs, we remount the root as read-write.  Therefore, we no longer
-     * check for different values for fs_state between the root file 
+     * check for different values for fs_state between the root file
      * system and the rest of file systems.
      */
     if (!((sblock.fs_clean == FS_CLEAN || sblock.fs_clean == FS_OK))) {
@@ -1262,9 +1257,9 @@ numbers(yp)
 }
 #endif
 
-/* Convert a raw device name into a block device name. 
+/* Convert a raw device name into a block device name.
  * If the block device is not found, return the raw device name.
- * For HP and SUN, the returned value is not changed. For other 
+ * For HP and SUN, the returned value is not changed. For other
  * platforms it is changed (I see no rhyme or reason -jpm).
  */
 char *
@@ -1299,7 +1294,7 @@ unrawname(rawdev)
     return (rawdev);
 }
 
-/* Convert a block device name into a raw device name. 
+/* Convert a block device name into a raw device name.
  * If the block device is not found, return null
  */
 char *
