@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,29 +10,13 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
 #include <rx/xdr.h>
 #include <afs/afsint.h>
-#include <stdio.h>
-#ifdef AFS_PTHREAD_ENV
-#include <assert.h>
-#else /* AFS_PTHREAD_ENV */
-#include <afs/assert.h>
-#endif /* AFS_PTHREAD_ENV */
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <errno.h>
-#if defined(AFS_SUN5_ENV) || defined(AFS_NT40_ENV)
-#include <string.h>
-#else
-#include <strings.h>
-#endif
-#ifndef AFS_NT40_ENV
-#include <unistd.h>
-#endif
 
 #include <afs/afsutil.h>
-    
+
 #include "nfs.h"
 #include "lwp.h"
 #include "lock.h"
@@ -45,15 +29,7 @@
 #include "salvage.h"
 #include "daemon_com.h"
 #include "fssync.h"
-
-#ifdef O_LARGEFILE
-#define afs_stat	stat64
-#else /* !O_LARGEFILE */
-#define afs_stat	stat
-#endif /* !O_LARGEFILE */
-
-/*@printflike@*/ extern void Log(const char *format, ...);
-
+#include "common.h"
 
 struct Lock localLock;
 
@@ -78,7 +54,7 @@ NukeProc(struct ViceInodeInfo *ainfo, afs_uint32 avolid, void *arock)
 {
     struct ilist **allInodes = (struct ilist **)arock;
     struct ilist *ti;
-    register afs_int32 i;
+    afs_int32 i;
 
 #ifndef AFS_PTHREAD_ENV
     IOMGR_Poll();		/* poll so we don't kill the RPC connection */
@@ -115,23 +91,20 @@ int
 nuke(char *aname, afs_int32 avolid)
 {
     /* first process the partition containing this junk */
-    struct afs_stat tstat;
+    struct afs_stat_st tstat;
     struct ilist *ti, *ni, *li=NULL;
-    register afs_int32 code;
+    afs_int32 code;
     int i, forceSal;
     char wpath[100];
     char *lastDevComp;
     struct DiskPartition64 *dp;
 #ifdef AFS_NAMEI_ENV
-#ifdef AFS_NT40_ENV
-    char path[MAX_PATH];
-#else
     char *path;
+
     namei_t ufs_name;
-#endif
 #endif /* AFS_NAMEI_ENV */
 #ifndef AFS_NAMEI_ENV
-    char devName[64]
+    char devName[64];
 #endif /* !AFS_NAMEI_ENV */
     IHandle_t *fileH;
     struct ilist *allInodes = 0;
@@ -163,7 +136,7 @@ nuke(char *aname, afs_int32 avolid)
 	strcpy(devName, tfile);	/* save this from the static buffer */
     }
     /* aim lastDevComp at the 'foo' of '/dev/foo' */
-    lastDevComp = strrchr(devName, '/');
+    lastDevComp = strrchr(devName, OS_DIRSEPC);
     /* either points at slash, or there is no slash; adjust appropriately */
     if (lastDevComp)
 	lastDevComp++;
@@ -178,7 +151,7 @@ nuke(char *aname, afs_int32 avolid)
      * volume we're nuking.
      */
     code =
-	ListViceInodes(lastDevComp, aname, NULL, NukeProc, avolid, &forceSal,
+	ListViceInodes(lastDevComp, aname, INVALID_FD, NukeProc, avolid, &forceSal,
 		       0, wpath, &allInodes);
     if (code == 0) {
 	/* actually do the idecs now */
@@ -192,15 +165,14 @@ nuke(char *aname, afs_int32 avolid)
 #ifdef AFS_NT40_ENV
 		IH_INIT(fileH, (int)(*lastDevComp - 'A'), avolid,
 			ti->inode[i]);
-		nt_HandleToName(path, fileH);
 #else
 		IH_INIT(fileH, (int)volutil_GetPartitionID(aname), avolid,
 			ti->inode[i]);
+#endif /* AFS_NT40_ENV */
 		namei_HandleToName(&ufs_name, fileH);
 		path = ufs_name.n_path;
-#endif /* AFS_NT40_ENV */
 		IH_RELEASE(fileH);
-		if (unlink(path) < 0) {
+		if (OS_UNLINK(path) < 0) {
 		    Log("Nuke: Failed to remove %s\n", path);
 		}
 #else /* AFS_NAMEI_ENV */
