@@ -1,7 +1,7 @@
 /*
  * Copyright 2006-2008, Sine Nomine Associates and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -17,23 +17,11 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <sys/types.h>
-#include <stdio.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
-#include <time.h>
-#else
-#include <sys/param.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <sys/time.h>
 #endif
-#include <errno.h>
-#include <assert.h>
-#include <signal.h>
-#include <string.h>
 
 #include <rx/xdr.h>
 #include <afs/afsint.h>
@@ -47,10 +35,9 @@
 #include "vnode.h"
 #include "volume.h"
 #include "partition.h"
+#include "common.h"
 #include <rx/rx_queue.h>
 
-/*@printflike@*/ extern void Log(const char *format, ...);
-
 #ifdef AFS_DEMAND_ATTACH_FS
 /*
  * SALVSYNC is a feature specific to the demand attach fileserver
@@ -60,7 +47,7 @@ extern int LogLevel;
 extern int VInit;
 extern pthread_mutex_t vol_salvsync_mutex;
 
-static SYNC_client_state salvsync_client_state = 
+static SYNC_client_state salvsync_client_state =
     { -1,                     /* file descriptor */
       SALVSYNC_ENDPOINT_DECL, /* server endpoint */
       SALVSYNC_PROTO_VERSION, /* protocol version */
@@ -124,7 +111,7 @@ SALVSYNC_askSalv(SYNC_command * com, SYNC_response * res)
 }
 
 afs_int32
-SALVSYNC_SalvageVolume(VolumeId volume, char *partName, int command, int reason, 
+SALVSYNC_SalvageVolume(VolumeId volume, char *partName, int command, int reason,
 		       afs_uint32 prio, SYNC_response * res_in)
 {
     SYNC_command com;
@@ -164,7 +151,7 @@ SALVSYNC_SalvageVolume(VolumeId volume, char *partName, int command, int reason,
 }
 
 afs_int32
-SALVSYNC_LinkVolume(VolumeId parent, 
+SALVSYNC_LinkVolume(VolumeId parent,
 		    VolumeId clone,
 		    char * partName,
 		    SYNC_response * res_in)
