@@ -10,28 +10,15 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <sys/types.h>
-#include <string.h>
-#ifdef HAVE_STDINT_H
-# include <stdint.h>
-#endif
 #ifdef AFS_NT40_ENV
-#include <time.h>
-#include <fcntl.h>
 #include <windows.h>
 #include <WINNT/afsevent.h>
-#else
-#include <sys/time.h>
-#include <sys/file.h>
-#include <netinet/in.h>
-#include <unistd.h>
 #endif
+
 #include <rx/xdr.h>
 #include <afs/afsint.h>
-#include <stdio.h>
-#include <signal.h>
-#include <afs/afs_assert.h>
 #include <afs/prs_fs.h>
 #include <afs/nfs.h>
 #include <lwp.h>
@@ -52,20 +39,14 @@
 #include <afs/keys.h>
 #include <afs/dir.h>
 #include <ubik.h>
-#include <fcntl.h>
-#include <sys/stat.h>
-
-#include <errno.h>
 #include <afs/audit.h>
 #include <afs/afsutil.h>
 #include <lwp.h>
+
 #include "volser.h"
 #include "volint.h"
 #include "volser_internal.h"
 
-/*@printflike@*/ extern void Log(const char *format, ...);
-/*@printflike@*/ extern void Abort(const char *format, ...);
-
 #define VolserVersion "2.0"
 #define N_SECURITY_OBJECTS 3
 
@@ -87,7 +68,6 @@ int udpBufSize = 0;		/* UDP buffer size for receive */
 
 int rxBind = 0;
 int rxkadDisableDotCheck = 0;
-int DoPreserveVolumeStats = 0;
 
 #define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
 afs_uint32 SHostAddrs[ADDRSPERSITE];
@@ -97,14 +77,6 @@ afs_uint32 SHostAddrs[ADDRSPERSITE];
 			  exit(code);                             \
 		       }
 
-#if defined(AFS_PTHREAD_ENV)
-int
-threadNum(void)
-{
-    return (intptr_t)pthread_getspecific(rx_thread_id_key);
-}
-#endif
-
 static void
 MyBeforeProc(struct rx_call *acall)
 {
@@ -146,6 +118,7 @@ BKGLoop(void *unused)
     struct timeval tv;
     int loop = 0;
 
+    afs_pthread_setname_self("vol bkg");
     while (1) {
 	tv.tv_sec = GCWAKEUP;
 	tv.tv_usec = 0;
@@ -345,13 +318,6 @@ main(int argc, char **argv)
 		exit(1);
 	    }
 	    rxMaxMTU = atoi(argv[++code]);
-	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
-		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d invalid; must be between %d-%" AFS_SIZET_FMT "\n",
-		       rxMaxMTU, RX_MIN_PACKET_SIZE,
-		       RX_MAX_PACKET_DATA_SIZE);
-		exit(1);
-	    }
 	} else if (strcmp(argv[code], "-sleep") == 0) {
 	    sscanf(argv[++code], "%d/%d", &TTsleep, &TTrun);
 	    if ((TTsleep < 0) || (TTrun <= 0)) {
@@ -375,18 +341,6 @@ main(int argc, char **argv)
 	    rx_enablePeerRPCStats();
 	} else if (strcmp(argv[code], "-enable_process_stats") == 0) {
 	    rx_enableProcessRPCStats();
-	} else if (strcmp(argv[code], "-preserve-vol-stats") == 0) {
-	    DoPreserveVolumeStats = 1;
-	} else if (strcmp(argv[code], "-sync") == 0) {
-	    if ((code + 1) >= argc) {
-		printf("You have to specify -sync <sync_behavior>\n");
-		exit(1);
-	    }
-	    ih_PkgDefaults();
-	    if (ih_SetSyncBehavior(argv[++code])) {
-		printf("Invalid -sync value %s\n", argv[code]);
-		exit(1);
-	    }
 	}
 #ifndef AFS_NT40_ENV
 	else if (strcmp(argv[code], "-syslog") == 0) {
@@ -407,7 +361,6 @@ main(int argc, char **argv)
 		   "[-udpsize <size of socket buffer in bytes>] "
 		   "[-syslog[=FACILITY]] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
-		   "[-sync <always | delayed | onclose | never>] "
 		   "[-help]\n");
 #else
 	    printf("Usage: volserver [-log] [-p <number of processes>] "
@@ -415,7 +368,6 @@ main(int argc, char **argv)
 		   "[-nojumbo] [-jumbo] [-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals] "
 		   "[-udpsize <size of socket buffer in bytes>] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
-		   "[-sync <always | delayed | onclose | never>] "
 		   "[-help]\n");
 #endif
 	    VS_EXIT(1);
@@ -436,7 +388,7 @@ main(int argc, char **argv)
     InitErrTabs();
 
 #ifdef AFS_PTHREAD_ENV
-    SetLogThreadNumProgram( threadNum );
+    SetLogThreadNumProgram( rx_GetThreadNum );
 #endif
 
 #ifdef AFS_NT40_ENV
@@ -495,7 +447,10 @@ main(int argc, char **argv)
 	rx_SetNoJumbo();
     }
     if (rxMaxMTU != -1) {
-	rx_SetMaxMTU(rxMaxMTU);
+	if (rx_SetMaxMTU(rxMaxMTU) != 0) {
+	    fprintf(stderr, "rxMaxMTU %d is invalid\n", rxMaxMTU);
+	    VS_EXIT(1);
+	}
     }
     rx_GetIFInfo();
     rx_SetRxDeadTime(420);
@@ -527,7 +482,7 @@ main(int argc, char **argv)
 	VS_EXIT(1);
     }
     afsconf_GetKey(tdir, 999, &tkey);
-    afsconf_BuildServerSecurityObjects(tdir, 0, &securityClasses, &numClasses);
+    afsconf_BuildServerSecurityObjects(tdir, &securityClasses, &numClasses);
     if (securityClasses[0] == NULL)
 	Abort("rxnull_NewServerSecurityObject");
     service =
@@ -564,9 +519,6 @@ main(int argc, char **argv)
 
     LogCommandLine(argc, argv, "Volserver", VolserVersion, "Starting AFS",
 		   Log);
-    if (afsconf_GetLatestKey(tdir, NULL, NULL) == 0) {
-	LogDesWarning();
-    }
     if (TTsleep) {
 	Log("Will sleep %d second%s every %d second%s\n", TTsleep,
 	    (TTsleep > 1) ? "s" : "", TTrun + TTsleep,
